"""Module that compiles the base DLL on load."""
import os
import os.path as op
import sys
import json

from pyrevit import PyRevitException, EXEC_PARAMS, HOST_APP
import pyrevit.engine as eng
from pyrevit import framework
from pyrevit.framework import List, Array
from pyrevit import api
from pyrevit import labs
from pyrevit.compat import safe_strtype, NETCORE
from pyrevit import BIN_DIR, RUNTIME_DIR
from pyrevit import coreutils
from pyrevit.coreutils import assmutils
from pyrevit.coreutils import logger
from pyrevit.coreutils import appdata
from pyrevit.loader import HASH_CUTOFF_LENGTH
from pyrevit.userconfig import user_config
import pyrevit.extensions as exts

#pylint: disable=W0703,C0302,C0103
mlogger = logger.get_logger(__name__)


# C:\Windows\Microsoft.NET\Framework\

INTERFACE_TYPES_DIR = RUNTIME_DIR

DOTNET_DIR = op.join(os.getenv('windir'), 'Microsoft.NET', 'Framework')
DOTNET64_DIR = op.join(os.getenv('windir'), 'Microsoft.NET', 'Framework64')

DOTNET_SDK_DIR = op.join(os.getenv('programfiles(x86)'),
                            'Reference Assemblies',
                            'Microsoft', 'Framework', '.NETFramework')


try:
    # get sorted list of installed frawework paths
    DOTNET_FRAMEWORK_DIRS = sorted(
        [x for x in os.listdir(DOTNET_DIR)
            if x.startswith('v4.') and 'X' not in x], reverse=True)
except Exception as fw_err:
    DOTNET_FRAMEWORK_DIRS = []
    mlogger.debug('Dotnet Frawework is not installed. | %s', fw_err)

try:
    # get sorted list of installed frawework paths
    DOTNET64_FRAMEWORK_DIRS = sorted(
        [x for x in os.listdir(DOTNET64_DIR)
            if x.startswith('v4.') and 'X' not in x], reverse=True)
except Exception as fw_err:
    DOTNET64_FRAMEWORK_DIRS = []
    mlogger.debug('Dotnet64 Frawework is not installed. | %s', fw_err)

try:
    # get sorted list of installed frawework sdk paths
    DOTNET_TARGETPACK_DIRS = sorted(
        [x for x in os.listdir(DOTNET_SDK_DIR)
            if x.startswith('v4.') and 'X' not in x], reverse=True)
except Exception as dotnet_sdk_err:
    DOTNET_TARGETPACK_DIRS = []
    mlogger.debug('Dotnet SDK is not installed. | %s', dotnet_sdk_err)


# base classes for pyRevit commands --------------------------------------------
RUNTIME_NAMESPACE = 'PyRevitLabs.PyRevit.Runtime'

# template python command class
CMD_EXECUTOR_TYPE_NAME = '{}.{}'\
    .format(RUNTIME_NAMESPACE, 'ScriptCommand')

# template python command availability class
CMD_AVAIL_TYPE_NAME_EXTENDED = \
    coreutils.make_canonical_name(RUNTIME_NAMESPACE, 'ScriptCommandExtendedAvail')
CMD_AVAIL_TYPE_NAME_SELECTION = \
    coreutils.make_canonical_name(RUNTIME_NAMESPACE, 'ScriptCommandSelectionAvail')
CMD_AVAIL_TYPE_NAME_ZERODOC = \
    coreutils.make_canonical_name(RUNTIME_NAMESPACE, 'ScriptCommandZeroDocAvail')

CMD_AVAIL_NAME_POSTFIX = '-avail'

SOURCE_FILE_EXT = '.cs'
SOURCE_FILE_FILTER = r'(\.cs)'

# get and load the active Cpython engine
CPYTHON_ENGINE = user_config.get_active_cpython_engine()

# create a hash for the loader assembly
# this hash is calculated based on:
# - runtime csharp files
# - runtime engine version
# - cpython engine version
mlogger.debug('Building on IronPython engine: %s', EXEC_PARAMS.engine_ver)
BASE_TYPES_DIR_HASH = \
    coreutils.get_str_hash(
        coreutils.calculate_dir_hash(
            INTERFACE_TYPES_DIR, '', SOURCE_FILE_FILTER
        )
        + EXEC_PARAMS.engine_ver
        + str(CPYTHON_ENGINE.Version) if CPYTHON_ENGINE else "0"
        )[:HASH_CUTOFF_LENGTH]
RUNTIME_ASSM_FILE_ID = '{}_{}'\
    .format(BASE_TYPES_DIR_HASH, RUNTIME_NAMESPACE)

RUNTIME_ASSM_FILE = \
    op.join(BIN_DIR, "pyRevitLabs.PyRevit.Runtime.{}.dll".format(HOST_APP.version))

# taking the name of the generated data file and use it as assembly name
RUNTIME_ASSM_NAME = op.splitext(op.basename(RUNTIME_ASSM_FILE))[0]
mlogger.debug('Interface types assembly file is: %s', RUNTIME_ASSM_NAME)


def _get_source_files_in(source_files_path):
    source_files = {}
    for source_file in os.listdir(source_files_path):
        if op.splitext(source_file)[1].lower() == SOURCE_FILE_EXT:
            source_filepath = op.join(source_files_path, source_file)
            mlogger.debug('Source file found: %s', source_filepath)
            source_files[source_file] = source_filepath
    return source_files


def _get_source_files():
    source_files = []
    source_dir = op.dirname(__file__)
    mlogger.debug('Source files location: %s', source_dir)
    all_sources = _get_source_files_in(source_dir)

    version_source_dir = op.join(op.dirname(__file__), HOST_APP.version)
    if op.exists(version_source_dir):
        mlogger.debug('Version-specific Source files location: %s',
                      version_source_dir)
        version_sources = _get_source_files_in(version_source_dir)
        all_sources.update(version_sources)

    source_files = all_sources.values()
    mlogger.debug('Source files to be compiled: %s', source_files)
    return source_files


def _get_framework_module(fw_module, fw64=False):
    # start with the newest sdk folder and
    # work backwards trying to find the dll
    fw_dir = DOTNET64_DIR if fw64 else DOTNET_DIR
    fw_folders = DOTNET64_FRAMEWORK_DIRS if fw64 else DOTNET_FRAMEWORK_DIRS
    for fw_folder in fw_folders:
        fw_module_file = op.join(
            fw_dir,
            fw_folder,
            coreutils.make_canonical_name(fw_module,
                                          framework.ASSEMBLY_FILE_TYPE))
        mlogger.debug('Searching for installed: %s', fw_module_file)
        if op.exists(fw_module_file):
            mlogger.debug('Found installed: %s', fw_module_file)
            sys.path.append(op.join(fw_dir, fw_folder))
            return fw_module_file

    return None


def _get_framework_sdk_module(fw_module):
    # start with the newest sdk folder and
    # work backwards trying to find the dll
    for sdk_folder in DOTNET_TARGETPACK_DIRS:
        fw_module_file = op.join(
            DOTNET_SDK_DIR,
            sdk_folder,
            coreutils.make_canonical_name(fw_module,
                                          framework.ASSEMBLY_FILE_TYPE))
        mlogger.debug('Searching for sdk: %s', fw_module_file)
        if op.exists(fw_module_file):
            mlogger.debug('Found sdk: %s', fw_module_file)
            sys.path.append(op.join(DOTNET_SDK_DIR, sdk_folder))
            return fw_module_file

    return None


def _get_reference_file(ref_name):
    mlogger.debug('Searching for dependency: %s', ref_name)
    # First try to find the dll in the project folder
    addin_file = framework.get_dll_file(ref_name)
    if addin_file:
        assmutils.load_asm_file(addin_file)
        return addin_file

    mlogger.debug('Dependency is not shipped: %s', ref_name)
    mlogger.debug('Searching for dependency in loaded assemblies: %s', ref_name)

    # Lastly try to find location of assembly if already loaded
    loaded_asm = assmutils.find_loaded_asm(ref_name)
    if loaded_asm:
        return loaded_asm[0].Location

    mlogger.debug('Dependency is not loaded: %s', ref_name)
    mlogger.debug('Searching for dependency in installed frameworks: %s',
                  ref_name)

    # Then try to find the dll in windows installed framework64 files
    if DOTNET64_DIR:
        fw_module_file = _get_framework_module(ref_name, fw64=True)
        if fw_module_file:
            return fw_module_file

    # Then try to find the dll in windows installed framework files
    if DOTNET_DIR:
        fw_module_file = _get_framework_module(ref_name)
        if fw_module_file:
            return fw_module_file

    mlogger.debug('Dependency is not installed: %s', ref_name)
    mlogger.debug('Searching for dependency in installed frameworks sdks: %s',
                  ref_name)

    # Then try to find the dll in windows SDK
    if DOTNET_TARGETPACK_DIRS:
        fw_sdk_module_file = _get_framework_sdk_module(ref_name)
        if fw_sdk_module_file:
            return fw_sdk_module_file

    # if not worked raise critical error
    mlogger.critical('Can not find required reference assembly: %s', ref_name)


def get_references():
    """Get list of all referenced assemblies.

    Returns:
        (list): referenced assemblies
    """
    ref_list = [
        # system stuff
        'System', 'System.Core', 'System.Runtime', 'System.Linq', 'System.Collections',
        'System.Collections.Immutable', 'System.Console',
        'System.Xaml', 'System.Web', 'System.Xml', 'System.Numerics',
        'System.Drawing', 'System.Windows.Forms',
        'System.ComponentModel.Primitives', 'System.ComponentModel.TypeConverter',
        'PresentationCore', 'PresentationFramework',
        'WindowsBase', 'WindowsFormsIntegration',
        # legacy csharp/vb.net compiler
        'Microsoft.CSharp',
        # iron python engine
        '{prefix}Microsoft.Dynamic'.format(prefix=eng.EnginePrefix),
        '{prefix}Microsoft.Scripting'.format(prefix=eng.EnginePrefix),
        '{prefix}IronPython'.format(prefix=eng.EnginePrefix),
        '{prefix}IronPython.Modules'.format(prefix=eng.EnginePrefix),
        # revit api
        'RevitAPI', 'RevitAPIUI', 'AdWindows', 'UIFramework',
        # pyrevit loader assembly
        'pyRevitLoader',
        # pyrevit labs
        'pyRevitLabs.Common', 'pyRevitLabs.CommonWPF',
        'pyRevitLabs.MahAppsMetro',
        'pyRevitLabs.NLog',
        'pyRevitLabs.Json',
        'pyRevitLabs.Emojis',
        'pyRevitLabs.TargetApps.Revit',
        'pyRevitLabs.PyRevit',
        'pyRevitLabs.PyRevit.Runtime.Shared',
    ]

    # netcore depends
    if NETCORE:
        ref_list.extend(['System.Drawing.Common',
                         'System.Diagnostics.Process',
                         'System.Diagnostics.FileVersionInfo',
                         'System.Text.RegularExpressions'])

    # another revit api
    if HOST_APP.is_newer_than(2018):
        ref_list.extend(['Xceed.Wpf.AvalonDock'])

    refs = (_get_reference_file(ref_name) for ref_name in ref_list)
    return [r for r in refs if r]


def _generate_runtime_asm():
    source_list = list(_get_source_files())
    # now try to load compiled runtime assembly
    try:
        return assmutils.load_asm_file(RUNTIME_ASSM_FILE)
    except PyRevitException as compile_err:
        errors = safe_strtype(compile_err).replace('Compile error: ', '')
        mlogger.critical('Can not compile base types code into assembly.\n%s',
                         errors)
        raise compile_err


def _get_runtime_asm():
    if appdata.is_data_file_available(file_id=RUNTIME_ASSM_FILE_ID,
                                      file_ext=framework.ASSEMBLY_FILE_TYPE):
        return assmutils.load_asm_file(RUNTIME_ASSM_FILE)
    else:
        return _generate_runtime_asm()


def create_ipyengine_configs(clean=False, full_frame=False, persistent=False):
    """Return the configuration for ipython engine.

    Args:
        clean (bool, optional): Engine should be clean. Defaults to False.
        full_frame (bool, optional): Engine shoul be full frame. Defaults to False.
        persistent (bool, optional): Engine should persist. Defaults to False.

    Returns:
        (str): Configuration
    """
    return json.dumps({
        exts.MDATA_ENGINE_CLEAN: clean,
        exts.MDATA_ENGINE_FULLFRAME: full_frame,
        exts.MDATA_ENGINE_PERSISTENT: persistent,
    })


def create_ext_command_attrs():
    """Create dotnet attributes for Revit external commands.

    This method is used in creating custom dotnet types for pyRevit commands
    and compiling them into a DLL assembly. Current implementation sets
    ``RegenerationOption.Manual`` and ``TransactionMode.Manual``

    Returns:
        (list[CustomAttributeBuilder]): object for `RegenerationOption` 
            and `TransactionMode` attributes.
    """
    regen_const_info = \
        framework.clr.GetClrType(api.Attributes.RegenerationAttribute) \
        .GetConstructor(
            framework.Array[framework.Type](
                (api.Attributes.RegenerationOption,)
                ))

    regen_attr_builder = \
        framework.CustomAttributeBuilder(
            regen_const_info,
            framework.Array[object](
                (api.Attributes.RegenerationOption.Manual,)
                ))

    # add TransactionAttribute to framework.Type
    trans_constructor_info = \
        framework.clr.GetClrType(api.Attributes.TransactionAttribute) \
        .GetConstructor(
            framework.Array[framework.Type](
                (api.Attributes.TransactionMode,)
                )
            )

    trans_attrib_builder = \
        framework.CustomAttributeBuilder(
            trans_constructor_info,
            framework.Array[object](
                (api.Attributes.TransactionMode.Manual,)
                )
            )

    return [regen_attr_builder, trans_attrib_builder]


def create_type(modulebuilder, type_class, class_name, custom_attr_list, *args):
    """Create a dotnet type for a pyRevit command.

    See ``baseclasses.cs`` code for the template pyRevit command dotnet type
    and its constructor default arguments that must be provided here.

    Args:
        modulebuilder (:obj:`ModuleBuilder`): dotnet module builder
        type_class (type): source dotnet type for the command
        class_name (str): name for the new type
        custom_attr_list (:obj:`list`): list of dotnet attributes for the type
        *args (Any): list of arguments to be used with type constructor

    Returns:
        (type): returns created dotnet type

    Examples:
        ```python
        asm_builder = AppDomain.CurrentDomain.DefineDynamicAssembly(
        win_asm_name, AssemblyBuilderAccess.RunAndSave, filepath
        )
        module_builder = asm_builder.DefineDynamicModule(
        ext_asm_file_name, ext_asm_full_file_name
        )
        create_type(
            module_builder,
            runtime.ScriptCommand,
            "PyRevitSomeCommandUniqueName",
            runtime.create_ext_command_attrs(),
            [scriptpath, atlscriptpath, searchpath, helpurl, name,
            bundle, extension, uniquename, False, False])
        ```
        <type PyRevitSomeCommandUniqueName>
    """
    # create type builder
    type_builder = \
        modulebuilder.DefineType(
            class_name,
            framework.TypeAttributes.Class | framework.TypeAttributes.Public,
            type_class
            )

    for custom_attr in custom_attr_list:
        type_builder.SetCustomAttribute(custom_attr)

    # prepare a list of input param types to find the matching constructor
    type_list = []
    param_list = []
    for param in args:
        if isinstance(param, str) \
                or isinstance(param, int):
            type_list.append(type(param))
            param_list.append(param)

    # call base constructor
    constructor = \
        type_class.GetConstructor(framework.Array[framework.Type](type_list))
    # create class constructor builder
    const_builder = \
        type_builder.DefineConstructor(framework.MethodAttributes.Public,
                                       framework.CallingConventions.Standard,
                                       framework.Array[framework.Type](()))
    # add constructor parameters to stack
    gen = const_builder.GetILGenerator()
    gen.Emit(framework.OpCodes.Ldarg_0)  # Load "this" onto eval stack

    # add constructor input params to the stack
    for param_type, param in zip(type_list, param_list):
        if param_type == str:
            gen.Emit(framework.OpCodes.Ldstr, param)
        elif param_type == int:
            gen.Emit(framework.OpCodes.Ldc_I4, param)

    # call base constructor (consumes "this" and the created stack)
    gen.Emit(framework.OpCodes.Call, constructor)
    # Fill some space - this is how it is generated for equivalent C# code
    gen.Emit(framework.OpCodes.Nop)
    gen.Emit(framework.OpCodes.Nop)
    gen.Emit(framework.OpCodes.Nop)
    gen.Emit(framework.OpCodes.Ret)
    return type_builder.CreateType()


# compile or load the base types assembly
# see it the assembly is already loaded
RUNTIME_ASSM = None
assm_list = assmutils.find_loaded_asm(RUNTIME_ASSM_NAME)
if assm_list:
    RUNTIME_ASSM = assm_list[0]
else:
    # else, let's generate the assembly and load it
    RUNTIME_ASSM = _get_runtime_asm()

if RUNTIME_ASSM is None:
    raise Exception("Error dynamically compiling pyRevit runtime")

CMD_EXECUTOR_TYPE = \
    assmutils.find_type_by_name(RUNTIME_ASSM, CMD_EXECUTOR_TYPE_NAME)

CMD_AVAIL_TYPE_EXTENDED = \
        assmutils.find_type_by_name(RUNTIME_ASSM,
                                    CMD_AVAIL_TYPE_NAME_EXTENDED)
CMD_AVAIL_TYPE_SELECTION = \
        assmutils.find_type_by_name(RUNTIME_ASSM,
                                    CMD_AVAIL_TYPE_NAME_SELECTION)
CMD_AVAIL_TYPE_ZERODOC = \
    assmutils.find_type_by_name(RUNTIME_ASSM,
                                CMD_AVAIL_TYPE_NAME_ZERODOC)



class RuntimeTypeHandle(object,ISerializable):
 """ Represents a type using an internal metadata token. """
 def Equals(self,*__args):
  """
  Equals(self: RuntimeTypeHandle,handle: RuntimeTypeHandle) -> bool

  

   Indicates whether the specified System.RuntimeTypeHandle structure is equal to the current 

    System.RuntimeTypeHandle structure.

  

  

   handle: The System.RuntimeTypeHandle structure to compare to the current instance.

   Returns: true if the value of handle is equal to the value of this instance; otherwise,false.

  Equals(self: RuntimeTypeHandle,obj: object) -> bool

  

   Indicates whether the specified object is equal to the current System.RuntimeTypeHandle 

    structure.

  

  

   obj: An object to compare to the current instance.

   Returns: true if obj is a System.RuntimeTypeHandle structure and is equal to the value of this instance; 

    otherwise,false.
  """
  pass
 def GetHashCode(self):
  """
  GetHashCode(self: RuntimeTypeHandle) -> int

  

   Returns the hash code for the current instance.

   Returns: A 32-bit signed integer hash code.
  """
  pass
 def GetModuleHandle(self):
  """
  GetModuleHandle(self: RuntimeTypeHandle) -> ModuleHandle

  

   Gets a handle to the module that contains the type represented by the current instance.

   Returns: A System.ModuleHandle structure representing a handle to the module that contains the type 

    represented by the current instance.
  """
  pass
 def GetObjectData(self,info,context):
  """
  GetObjectData(self: RuntimeTypeHandle,info: SerializationInfo,context: StreamingContext)

   Populates a System.Runtime.Serialization.SerializationInfo with the data necessary to 

    deserialize the type represented by the current instance.

  

  

   info: The object to be populated with serialization information.

   context: (Reserved) The location where serialized data will be stored and retrieved.
  """
  pass
 def __eq__(self,*args):
  """ x.__eq__(y) <==> x==y """
  pass
 def __init__(self,*args):
  """ x.__init__(...) initializes x; see x.__class__.__doc__ for signaturex.__init__(...) initializes x; see x.__class__.__doc__ for signaturex.__init__(...) initializes x; see x.__class__.__doc__ for signature """
  pass
 def __ne__(self,*args):
  pass
 def __reduce_ex__(self,*args):
  pass
 def __repr__(self,*args):
  """ __repr__(self: object) -> str """
  pass
 def __str__(self,*args):
  pass
 Value=property(lambda self: object(),lambda self,v: None,lambda self: None)
 """Gets a handle to the type represented by this instance.



Get: Value(self: RuntimeTypeHandle) -> IntPtr



"""





class BackgroundWorker(Component,IComponent,IDisposable):
 """
 Executes an operation on a separate thread.

 

 BackgroundWorker()
 """
 def CancelAsync(self):
  """
  CancelAsync(self: BackgroundWorker)

   Requests cancellation of a pending background operation.
  """
  pass
 def Dispose(self):
  """
  Dispose(self: Component,disposing: bool)

   Releases the unmanaged resources used by the System.ComponentModel.Component and optionally 

    releases the managed resources.

  

  

   disposing: true to release both managed and unmanaged resources; false to release only unmanaged resources.
  """
  pass
 def GetService(self,*args):
  """
  GetService(self: Component,service: Type) -> object

  

   Returns an object that represents a service provided by the System.ComponentModel.Component or 

    by its System.ComponentModel.Container.

  

  

   service: A service provided by the System.ComponentModel.Component.

   Returns: An System.Object that represents a service provided by the System.ComponentModel.Component,or 

    null if the System.ComponentModel.Component does not provide the specified service.
  """
  pass
 def MemberwiseClone(self,*args):
  """
  MemberwiseClone(self: MarshalByRefObject,cloneIdentity: bool) -> MarshalByRefObject

  

   Creates a shallow copy of the current System.MarshalByRefObject object.

  

   cloneIdentity: false to delete the current System.MarshalByRefObject object's identity,which will cause the 

    object to be assigned a new identity when it is marshaled across a remoting boundary. A value of 

    false is usually appropriate. true to copy the current System.MarshalByRefObject object's 

    identity to its clone,which will cause remoting client calls to be routed to the remote server 

    object.

  

   Returns: A shallow copy of the current System.MarshalByRefObject object.

  MemberwiseClone(self: object) -> object

  

   Creates a shallow copy of the current System.Object.

   Returns: A shallow copy of the current System.Object.
  """
  pass
 def OnDoWork(self,*args):
  """
  OnDoWork(self: BackgroundWorker,e: DoWorkEventArgs)

   Raises the System.ComponentModel.BackgroundWorker.DoWork event.

  

   e: An System.EventArgs that contains the event data.
  """
  pass
 def OnProgressChanged(self,*args):
  """
  OnProgressChanged(self: BackgroundWorker,e: ProgressChangedEventArgs)

   Raises the System.ComponentModel.BackgroundWorker.ProgressChanged event.

  

   e: An System.EventArgs that contains the event data.
  """
  pass
 def OnRunWorkerCompleted(self,*args):
  """
  OnRunWorkerCompleted(self: BackgroundWorker,e: RunWorkerCompletedEventArgs)

   Raises the System.ComponentModel.BackgroundWorker.RunWorkerCompleted event.

  

   e: An System.EventArgs that contains the event data.
  """
  pass
 def ReportProgress(self,percentProgress,userState=None):
  """
  ReportProgress(self: BackgroundWorker,percentProgress: int,userState: object)

   Raises the System.ComponentModel.BackgroundWorker.ProgressChanged event.

  

   percentProgress: The percentage,from 0 to 100,of the background operation that is complete.

   userState: The state object passed to System.ComponentModel.BackgroundWorker.RunWorkerAsync(System.Object).

  ReportProgress(self: BackgroundWorker,percentProgress: int)

   Raises the System.ComponentModel.BackgroundWorker.ProgressChanged event.

  

   percentProgress: The percentage,from 0 to 100,of the background operation that is complete.
  """
  pass
 def RunWorkerAsync(self,argument=None):
  """
  RunWorkerAsync(self: BackgroundWorker,argument: object)

   Starts execution of a background operation.

  

   argument: A parameter for use by the background operation to be executed in the 

    System.ComponentModel.BackgroundWorker.DoWork event handler.

  

  RunWorkerAsync(self: BackgroundWorker)

   Starts execution of a background operation.
  """
  pass
 def __enter__(self,*args):
  """
  __enter__(self: IDisposable) -> object

  

   Provides the implementation of __enter__ for objects which implement IDisposable.
  """
  pass
 def __exit__(self,*args):
  """
  __exit__(self: IDisposable,exc_type: object,exc_value: object,exc_back: object)

   Provides the implementation of __exit__ for objects which implement IDisposable.
  """
  pass
 def __init__(self,*args):
  """ x.__init__(...) initializes x; see x.__class__.__doc__ for signaturex.__init__(...) initializes x; see x.__class__.__doc__ for signaturex.__init__(...) initializes x; see x.__class__.__doc__ for signature """
  pass
 def __str__(self,*args):
  pass
 CancellationPending=property(lambda self: object(),lambda self,v: None,lambda self: None)
 """Gets a value indicating whether the application has requested cancellation of a background operation.



Get: CancellationPending(self: BackgroundWorker) -> bool



"""

 CanRaiseEvents=property(lambda self: object(),lambda self,v: None,lambda self: None)
 """Gets a value indicating whether the component can raise an event.



"""

 DesignMode=property(lambda self: object(),lambda self,v: None,lambda self: None)
 """Gets a value that indicates whether the System.ComponentModel.Component is currently in design mode.



"""

 Events=property(lambda self: object(),lambda self,v: None,lambda self: None)
 """Gets the list of event handlers that are attached to this System.ComponentModel.Component.



"""

 IsBusy=property(lambda self: object(),lambda self,v: None,lambda self: None)
 """Gets a value indicating whether the System.ComponentModel.BackgroundWorker is running an asynchronous operation.



Get: IsBusy(self: BackgroundWorker) -> bool



"""

 WorkerReportsProgress=property(lambda self: object(),lambda self,v: None,lambda self: None)
 """Gets or sets a value indicating whether the System.ComponentModel.BackgroundWorker can report progress updates.



Get: WorkerReportsProgress(self: BackgroundWorker) -> bool



Set: WorkerReportsProgress(self: BackgroundWorker)=value

"""

 WorkerSupportsCancellation=property(lambda self: object(),lambda self,v: None,lambda self: None)
 """Gets or sets a value indicating whether the System.ComponentModel.BackgroundWorker supports asynchronous cancellation.



Get: WorkerSupportsCancellation(self: BackgroundWorker) -> bool



Set: WorkerSupportsCancellation(self: BackgroundWorker)=value

"""


 DoWork=None
 ProgressChanged=None
 RunWorkerCompleted=None




# encoding: utf-8
# module System.Drawing.Design calls itself Design
# from System.Drawing,Version=4.0.0.0,Culture=neutral,PublicKeyToken=b03f5f7f11d50a3a
# by generator 1.145
""" NamespaceTracker represent a CLS namespace. """
# no imports

# no functions
# classes

class CategoryNameCollection(ReadOnlyCollectionBase,ICollection,IEnumerable):
 """
 Represents a collection of category name strings.

 

 CategoryNameCollection(value: CategoryNameCollection)

 CategoryNameCollection(value: Array[str])
 """
 def Contains(self,value):
  """
  Contains(self: CategoryNameCollection,value: str) -> bool

  

   Indicates whether the specified category is contained in the collection.

  

   value: The string to check for in the collection.

   Returns: true if the specified category is contained in the collection; otherwise,false.
  """
  pass
 def CopyTo(self,array,index):
  """
  CopyTo(self: CategoryNameCollection,array: Array[str],index: int)

   Copies the collection elements to the specified array at the specified index.

  

   array: The array to copy to.

   index: The index of the destination array at which to begin copying.
  """
  pass
 def IndexOf(self,value):
  """
  IndexOf(self: CategoryNameCollection,value: str) -> int

  

   Gets the index of the specified value.

  

   value: The category name to retrieve the index of in the collection.

   Returns: The index in the collection,or null if the string does not exist in the collection.
  """
  pass
 def __getitem__(self,*args):
  """ x.__getitem__(y) <==> x[y] """
  pass
 def __init__(self,*args):
  """ x.__init__(...) initializes x; see x.__class__.__doc__ for signaturex.__init__(...) initializes x; see x.__class__.__doc__ for signaturex.__init__(...) initializes x; see x.__class__.__doc__ for signature """
  pass
 def __iter__(self,*args):
  """ __iter__(self: IEnumerable) -> object """
  pass
 @staticmethod
 def __new__(self,value):
  """
  __new__(cls: type,value: CategoryNameCollection)

  __new__(cls: type,value: Array[str])
  """
  pass
 InnerList=property(lambda self: object(),lambda self,v: None,lambda self: None)
 """Gets the list of elements contained in the System.Collections.ReadOnlyCollectionBase instance.



"""



class IPropertyValueUIService:
 """ Provides an interface to manage the images,ToolTips,and event handlers for the properties of a component displayed in a property browser. """
 def AddPropertyValueUIHandler(self,newHandler):
  """
  AddPropertyValueUIHandler(self: IPropertyValueUIService,newHandler: PropertyValueUIHandler)

   Adds the specified System.Drawing.Design.PropertyValueUIHandler to this service.

  

   newHandler: The property value UI handler to add.
  """
  pass
 def GetPropertyUIValueItems(self,context,propDesc):
  """
  GetPropertyUIValueItems(self: IPropertyValueUIService,context: ITypeDescriptorContext,propDesc: PropertyDescriptor) -> Array[PropertyValueUIItem]

  

   Gets the System.Drawing.Design.PropertyValueUIItem objects that match the specified context and 

    property descriptor characteristics.

  

  

   context: An System.ComponentModel.ITypeDescriptorContext that can be used to gain additional context 

    information.

  

   propDesc: A System.ComponentModel.PropertyDescriptor that indicates the property to match with the 

    properties to return.

  

   Returns: An array of System.Drawing.Design.PropertyValueUIItem objects that match the specified 

    parameters.
  """
  pass
 def NotifyPropertyValueUIItemsChanged(self):
  """
  NotifyPropertyValueUIItemsChanged(self: IPropertyValueUIService)

   Notifies the System.Drawing.Design.IPropertyValueUIService implementation that the global list 

    of System.Drawing.Design.PropertyValueUIItem objects has been modified.
  """
  pass
 def RemovePropertyValueUIHandler(self,newHandler):
  """
  RemovePropertyValueUIHandler(self: IPropertyValueUIService,newHandler: PropertyValueUIHandler)

   Removes the specified System.Drawing.Design.PropertyValueUIHandler from the property value UI 

    service.

  

  

   newHandler: The handler to remove.
  """
  pass
 def __init__(self,*args):
  """ x.__init__(...) initializes x; see x.__class__.__doc__ for signaturex.__init__(...) initializes x; see x.__class__.__doc__ for signature """
  pass
 PropertyUIValueItemsChanged=None


class IToolboxItemProvider:
 """ Exposes a collection of toolbox items. """
 def __init__(self,*args):
  """ x.__init__(...) initializes x; see x.__class__.__doc__ for signaturex.__init__(...) initializes x; see x.__class__.__doc__ for signature """
  pass
 Items=property(lambda self: object(),lambda self,v: None,lambda self: None)
 """Gets a collection of System.Drawing.Design.ToolboxItem objects.



Get: Items(self: IToolboxItemProvider) -> ToolboxItemCollection



"""



class IToolboxService:
 """ Provides methods and properties to manage and query the toolbox in the development environment. """
 def AddCreator(self,creator,format,host=None):
  """
  AddCreator(self: IToolboxService,creator: ToolboxItemCreatorCallback,format: str,host: IDesignerHost)

   Adds a new toolbox item creator for a specified data format and designer host.

  

   creator: A System.Drawing.Design.ToolboxItemCreatorCallback that can create a component when the toolbox 

    item is invoked.

  

   format: The data format that the creator handles.

   host: The System.ComponentModel.Design.IDesignerHost that represents the designer host to associate 

    with the creator.

  

  AddCreator(self: IToolboxService,creator: ToolboxItemCreatorCallback,format: str)

   Adds a new toolbox item creator for a specified data format.

  

   creator: A System.Drawing.Design.ToolboxItemCreatorCallback that can create a component when the toolbox 

    item is invoked.

  

   format: The data format that the creator handles.
  """
  pass
 def AddLinkedToolboxItem(self,toolboxItem,*__args):
  """
  AddLinkedToolboxItem(self: IToolboxService,toolboxItem: ToolboxItem,category: str,host: IDesignerHost)

   Adds the specified project-linked toolbox item to the toolbox in the specified category.

  

   toolboxItem: The linked System.Drawing.Design.ToolboxItem to add to the toolbox.

   category: The toolbox item category to add the toolbox item to.

   host: The System.ComponentModel.Design.IDesignerHost for the current design document.

  AddLinkedToolboxItem(self: IToolboxService,toolboxItem: ToolboxItem,host: IDesignerHost)

   Adds the specified project-linked toolbox item to the toolbox.

  

   toolboxItem: The linked System.Drawing.Design.ToolboxItem to add to the toolbox.

   host: The System.ComponentModel.Design.IDesignerHost for the current design document.
  """
  pass
 def AddToolboxItem(self,toolboxItem,category=None):
  """
  AddToolboxItem(self: IToolboxService,toolboxItem: ToolboxItem,category: str)

   Adds the specified toolbox item to the toolbox in the specified category.

  

   toolboxItem: The System.Drawing.Design.ToolboxItem to add to the toolbox.

   category: The toolbox item category to add the System.Drawing.Design.ToolboxItem to.

  AddToolboxItem(self: IToolboxService,toolboxItem: ToolboxItem)

   Adds the specified toolbox item to the toolbox.

  

   toolboxItem: The System.Drawing.Design.ToolboxItem to add to the toolbox.
  """
  pass
 def DeserializeToolboxItem(self,serializedObject,host=None):
  """
  DeserializeToolboxItem(self: IToolboxService,serializedObject: object,host: IDesignerHost) -> ToolboxItem

  

   Gets a toolbox item from the specified object that represents a toolbox item in serialized form,

    using the specified designer host.

  

  

   serializedObject: The object that contains the System.Drawing.Design.ToolboxItem to retrieve.

   host: The System.ComponentModel.Design.IDesignerHost to associate with this 

    System.Drawing.Design.ToolboxItem.

  

   Returns: The System.Drawing.Design.ToolboxItem created from deserialization.

  DeserializeToolboxItem(self: IToolboxService,serializedObject: object) -> ToolboxItem

  

   Gets a toolbox item from the specified object that represents a toolbox item in serialized form.

  

   serializedObject: The object that contains the System.Drawing.Design.ToolboxItem to retrieve.

   Returns: The System.Drawing.Design.ToolboxItem created from the serialized object.
  """
  pass
 def GetSelectedToolboxItem(self,host=None):
  """
  GetSelectedToolboxItem(self: IToolboxService,host: IDesignerHost) -> ToolboxItem

  

   Gets the currently selected toolbox item if it is available to all designers,or if it supports 

    the specified designer.

  

  

   host: The System.ComponentModel.Design.IDesignerHost that the selected tool must be associated with 

    for it to be returned.

  

   Returns: The System.Drawing.Design.ToolboxItem that is currently selected,or null if no toolbox item is 

    currently selected.

  

  GetSelectedToolboxItem(self: IToolboxService) -> ToolboxItem

  

   Gets the currently selected toolbox item.

   Returns: The System.Drawing.Design.ToolboxItem that is currently selected,or null if no toolbox item has 

    been selected.
  """
  pass
 def GetToolboxItems(self,*__args):
  """
  GetToolboxItems(self: IToolboxService,category: str) -> ToolboxItemCollection

  

   Gets a collection of toolbox items from the toolbox that match the specified category.

  

   category: The toolbox item category to retrieve all the toolbox items from.

   Returns: A System.Drawing.Design.ToolboxItemCollection that contains the current toolbox items that are 

    associated with the specified category.

  

  GetToolboxItems(self: IToolboxService,category: str,host: IDesignerHost) -> ToolboxItemCollection

  

   Gets the collection of toolbox items that are associated with the specified designer host and 

    category from the toolbox.

  

  

   category: The toolbox item category to retrieve the toolbox items from.

   host: The System.ComponentModel.Design.IDesignerHost that is associated with the toolbox items to 

    retrieve.

  

   Returns: A System.Drawing.Design.ToolboxItemCollection that contains the current toolbox items that are 

    associated with the specified category and designer host.

  

  GetToolboxItems(self: IToolboxService) -> ToolboxItemCollection

  

   Gets the entire collection of toolbox items from the toolbox.

   Returns: A System.Drawing.Design.ToolboxItemCollection that contains the current toolbox items.

  GetToolboxItems(self: IToolboxService,host: IDesignerHost) -> ToolboxItemCollection

  

   Gets the collection of toolbox items that are associated with the specified designer host from 

    the toolbox.

  

  

   host: The System.ComponentModel.Design.IDesignerHost that is associated with the toolbox items to 

    retrieve.

  

   Returns: A System.Drawing.Design.ToolboxItemCollection that contains the current toolbox items that are 

    associated with the specified designer host.
  """
  pass
 def IsSupported(self,serializedObject,*__args):
  """
  IsSupported(self: IToolboxService,serializedObject: object,filterAttributes: ICollection) -> bool

  

   Gets a value indicating whether the specified object which represents a serialized toolbox item 

    matches the specified attributes.

  

  

   serializedObject: The object that contains the System.Drawing.Design.ToolboxItem to retrieve.

   filterAttributes: An System.Collections.ICollection that contains the attributes to test the serialized object for.

   Returns: true if the object matches the specified attributes; otherwise,false.

  IsSupported(self: IToolboxService,serializedObject: object,host: IDesignerHost) -> bool

  

   Gets a value indicating whether the specified object which represents a serialized toolbox item 

    can be used by the specified designer host.

  

  

   serializedObject: The object that contains the System.Drawing.Design.ToolboxItem to retrieve.

   host: The System.ComponentModel.Design.IDesignerHost to test for support for the 

    System.Drawing.Design.ToolboxItem.

  

   Returns: true if the specified object is compatible with the specified designer host; otherwise,false.
  """
  pass
 def IsToolboxItem(self,serializedObject,host=None):
  """
  IsToolboxItem(self: IToolboxService,serializedObject: object,host: IDesignerHost) -> bool

  

   Gets a value indicating whether the specified object is a serialized toolbox item,using the 

    specified designer host.

  

  

   serializedObject: The object to inspect.

   host: The System.ComponentModel.Design.IDesignerHost that is making this request.

   Returns: true if the object contains a toolbox item object; otherwise,false.

  IsToolboxItem(self: IToolboxService,serializedObject: object) -> bool

  

   Gets a value indicating whether the specified object is a serialized toolbox item.

  

   serializedObject: The object to inspect.

   Returns: true if the object contains a toolbox item object; otherwise,false.
  """
  pass
 def Refresh(self):
  """
  Refresh(self: IToolboxService)

   Refreshes the state of the toolbox items.
  """
  pass
 def RemoveCreator(self,format,host=None):
  """
  RemoveCreator(self: IToolboxService,format: str,host: IDesignerHost)

   Removes a previously added toolbox creator that is associated with the specified data format and 

    the specified designer host.

  

  

   format: The data format of the creator to remove.

   host: The System.ComponentModel.Design.IDesignerHost that is associated with the creator to remove.

  RemoveCreator(self: IToolboxService,format: str)

   Removes a previously added toolbox item creator of the specified data format.

  

   format: The data format of the creator to remove.
  """
  pass
 def RemoveToolboxItem(self,toolboxItem,category=None):
  """
  RemoveToolboxItem(self: IToolboxService,toolboxItem: ToolboxItem,category: str)

   Removes the specified toolbox item from the toolbox.

  

   toolboxItem: The System.Drawing.Design.ToolboxItem to remove from the toolbox.

   category: The toolbox item category to remove the System.Drawing.Design.ToolboxItem from.

  RemoveToolboxItem(self: IToolboxService,toolboxItem: ToolboxItem)

   Removes the specified toolbox item from the toolbox.

  

   toolboxItem: The System.Drawing.Design.ToolboxItem to remove from the toolbox.
  """
  pass
 def SelectedToolboxItemUsed(self):
  """
  SelectedToolboxItemUsed(self: IToolboxService)

   Notifies the toolbox service that the selected tool has been used.
  """
  pass
 def SerializeToolboxItem(self,toolboxItem):
  """
  SerializeToolboxItem(self: IToolboxService,toolboxItem: ToolboxItem) -> object

  

   Gets a serializable object that represents the specified toolbox item.

  

   toolboxItem: The System.Drawing.Design.ToolboxItem to serialize.

   Returns: An object that represents the specified System.Drawing.Design.ToolboxItem.
  """
  pass
 def SetCursor(self):
  """
  SetCursor(self: IToolboxService) -> bool

  

   Sets the current application's cursor to a cursor that represents the currently selected tool.

   Returns: true if the cursor is set by the currently selected tool,false if there is no tool selected and 

    the cursor is set to the standard windows cursor.
  """
  pass
 def SetSelectedToolboxItem(self,toolboxItem):
  """
  SetSelectedToolboxItem(self: IToolboxService,toolboxItem: ToolboxItem)

   Selects the specified toolbox item.

  

   toolboxItem: The System.Drawing.Design.ToolboxItem to select.
  """
  pass
 def __init__(self,*args):
  """ x.__init__(...) initializes x; see x.__class__.__doc__ for signaturex.__init__(...) initializes x; see x.__class__.__doc__ for signature """
  pass
 CategoryNames=property(lambda self: object(),lambda self,v: None,lambda self: None)
 """Gets the names of all the tool categories currently on the toolbox.



Get: CategoryNames(self: IToolboxService) -> CategoryNameCollection



"""

 SelectedCategory=property(lambda self: object(),lambda self,v: None,lambda self: None)
 """Gets or sets the name of the currently selected tool category from the toolbox.



Get: SelectedCategory(self: IToolboxService) -> str



Set: SelectedCategory(self: IToolboxService)=value

"""



class IToolboxUser:
 """ Defines an interface for setting the currently selected toolbox item and indicating whether a designer supports a particular toolbox item. """
 def GetToolSupported(self,tool):
  """
  GetToolSupported(self: IToolboxUser,tool: ToolboxItem) -> bool

  

   Gets a value indicating whether the specified tool is supported by the current designer.

  

   tool: The System.Drawing.Design.ToolboxItem to be tested for toolbox support.

   Returns: true if the tool is supported by the toolbox and can be enabled; false if the document designer 

    does not know how to use the tool.
  """
  pass
 def ToolPicked(self,tool):
  """
  ToolPicked(self: IToolboxUser,tool: ToolboxItem)

   Selects the specified tool.

  

   tool: The System.Drawing.Design.ToolboxItem to select.
  """
  pass
 def __init__(self,*args):
  """ x.__init__(...) initializes x; see x.__class__.__doc__ for signaturex.__init__(...) initializes x; see x.__class__.__doc__ for signature """
  pass

class PaintValueEventArgs(EventArgs):
 """
 Provides data for the System.Drawing.Design.UITypeEditor.PaintValue(System.Object,System.Drawing.Graphics,System.Drawing.Rectangle) method.

 

 PaintValueEventArgs(context: ITypeDescriptorContext,value: object,graphics: Graphics,bounds: Rectangle)
 """
 @staticmethod
 def __new__(self,context,value,graphics,bounds):
  """ __new__(cls: type,context: ITypeDescriptorContext,value: object,graphics: Graphics,bounds: Rectangle) """
  pass
 Bounds=property(lambda self: object(),lambda self,v: None,lambda self: None)
 """Gets the rectangle that indicates the area in which the painting should be done.



Get: Bounds(self: PaintValueEventArgs) -> Rectangle



"""

 Context=property(lambda self: object(),lambda self,v: None,lambda self: None)
 """Gets the System.ComponentModel.ITypeDescriptorContext interface to be used to gain additional information about the context this value appears in.



Get: Context(self: PaintValueEventArgs) -> ITypeDescriptorContext



"""

 Graphics=property(lambda self: object(),lambda self,v: None,lambda self: None)
 """Gets the System.Drawing.Graphics object with which painting should be done.



Get: Graphics(self: PaintValueEventArgs) -> Graphics



"""

 Value=property(lambda self: object(),lambda self,v: None,lambda self: None)
 """Gets the value to paint.



Get: Value(self: PaintValueEventArgs) -> object



"""



class PropertyValueUIHandler(MulticastDelegate,ICloneable,ISerializable):
 """
 Represents the method that adds a delegate to an implementation of System.Drawing.Design.IPropertyValueUIService.

 

 PropertyValueUIHandler(object: object,method: IntPtr)
 """
 def BeginInvoke(self,context,propDesc,valueUIItemList,callback,object):
  """ BeginInvoke(self: PropertyValueUIHandler,context: ITypeDescriptorContext,propDesc: PropertyDescriptor,valueUIItemList: ArrayList,callback: AsyncCallback,object: object) -> IAsyncResult """
  pass
 def CombineImpl(self,*args):
  """
  CombineImpl(self: MulticastDelegate,follow: Delegate) -> Delegate

  

   Combines this System.Delegate with the specified System.Delegate to form a new delegate.

  

   follow: The delegate to combine with this delegate.

   Returns: A delegate that is the new root of the System.MulticastDelegate invocation list.
  """
  pass
 def DynamicInvokeImpl(self,*args):
  """
  DynamicInvokeImpl(self: Delegate,args: Array[object]) -> object

  

   Dynamically invokes (late-bound) the method represented by the current delegate.

  

   args: An array of objects that are the arguments to pass to the method represented by the current 

    delegate.-or- null,if the method represented by the current delegate does not require 

    arguments.

  

   Returns: The object returned by the method represented by the delegate.
  """
  pass
 def EndInvoke(self,result):
  """ EndInvoke(self: PropertyValueUIHandler,result: IAsyncResult) """
  pass
 def GetMethodImpl(self,*args):
  """
  GetMethodImpl(self: MulticastDelegate) -> MethodInfo

  

   Returns a static method represented by the current System.MulticastDelegate.

   Returns: A static method represented by the current System.MulticastDelegate.
  """
  pass
 def Invoke(self,context,propDesc,valueUIItemList):
  """ Invoke(self: PropertyValueUIHandler,context: ITypeDescriptorContext,propDesc: PropertyDescriptor,valueUIItemList: ArrayList) """
  pass
 def RemoveImpl(self,*args):
  """
  RemoveImpl(self: MulticastDelegate,value: Delegate) -> Delegate

  

   Removes an element from the invocation list of this System.MulticastDelegate that is equal to 

    the specified delegate.

  

  

   value: The delegate to search for in the invocation list.

   Returns: If value is found in the invocation list for this instance,then a new System.Delegate without 

    value in its invocation list; otherwise,this instance with its original invocation list.
  """
  pass
 def __init__(self,*args):
  """ x.__init__(...) initializes x; see x.__class__.__doc__ for signaturex.__init__(...) initializes x; see x.__class__.__doc__ for signaturex.__init__(...) initializes x; see x.__class__.__doc__ for signature """
  pass
 @staticmethod
 def __new__(self,object,method):
  """ __new__(cls: type,object: object,method: IntPtr) """
  pass
 def __reduce_ex__(self,*args):
  pass

class PropertyValueUIItem(object):
 """
 Provides information about a property displayed in the Properties window,including the associated event handler,pop-up information string,and the icon to display for the property.

 

 PropertyValueUIItem(uiItemImage: Image,handler: PropertyValueUIItemInvokeHandler,tooltip: str)
 """
 def Reset(self):
  """
  Reset(self: PropertyValueUIItem)

   Resets the user interface (UI) item.
  """
  pass
 @staticmethod
 def __new__(self,uiItemImage,handler,tooltip):
  """ __new__(cls: type,uiItemImage: Image,handler: PropertyValueUIItemInvokeHandler,tooltip: str) """
  pass
 Image=property(lambda self: object(),lambda self,v: None,lambda self: None)
 """Gets the 8 x 8 pixel image that will be drawn in the Properties window.



Get: Image(self: PropertyValueUIItem) -> Image



"""

 InvokeHandler=property(lambda self: object(),lambda self,v: None,lambda self: None)
 """Gets the handler that is raised when a user double-clicks this item.



Get: InvokeHandler(self: PropertyValueUIItem) -> PropertyValueUIItemInvokeHandler



"""

 ToolTip=property(lambda self: object(),lambda self,v: None,lambda self: None)
 """Gets or sets the information string to display for this item.



Get: ToolTip(self: PropertyValueUIItem) -> str



"""



class PropertyValueUIItemInvokeHandler(MulticastDelegate,ICloneable,ISerializable):
 """
 Represents the method that will handle the System.Drawing.Design.PropertyValueUIItem.InvokeHandler event of a System.Drawing.Design.PropertyValueUIItem.

 

 PropertyValueUIItemInvokeHandler(object: object,method: IntPtr)
 """
 def BeginInvoke(self,context,descriptor,invokedItem,callback,object):
  """ BeginInvoke(self: PropertyValueUIItemInvokeHandler,context: ITypeDescriptorContext,descriptor: PropertyDescriptor,invokedItem: PropertyValueUIItem,callback: AsyncCallback,object: object) -> IAsyncResult """
  pass
 def CombineImpl(self,*args):
  """
  CombineImpl(self: MulticastDelegate,follow: Delegate) -> Delegate

  

   Combines this System.Delegate with the specified System.Delegate to form a new delegate.

  

   follow: The delegate to combine with this delegate.

   Returns: A delegate that is the new root of the System.MulticastDelegate invocation list.
  """
  pass
 def DynamicInvokeImpl(self,*args):
  """
  DynamicInvokeImpl(self: Delegate,args: Array[object]) -> object

  

   Dynamically invokes (late-bound) the method represented by the current delegate.

  

   args: An array of objects that are the arguments to pass to the method represented by the current 

    delegate.-or- null,if the method represented by the current delegate does not require 

    arguments.

  

   Returns: The object returned by the method represented by the delegate.
  """
  pass
 def EndInvoke(self,result):
  """ EndInvoke(self: PropertyValueUIItemInvokeHandler,result: IAsyncResult) """
  pass
 def GetMethodImpl(self,*args):
  """
  GetMethodImpl(self: MulticastDelegate) -> MethodInfo

  

   Returns a static method represented by the current System.MulticastDelegate.

   Returns: A static method represented by the current System.MulticastDelegate.
  """
  pass
 def Invoke(self,context,descriptor,invokedItem):
  """ Invoke(self: PropertyValueUIItemInvokeHandler,context: ITypeDescriptorContext,descriptor: PropertyDescriptor,invokedItem: PropertyValueUIItem) """
  pass
 def RemoveImpl(self,*args):
  """
  RemoveImpl(self: MulticastDelegate,value: Delegate) -> Delegate

  

   Removes an element from the invocation list of this System.MulticastDelegate that is equal to 

    the specified delegate.

  

  

   value: The delegate to search for in the invocation list.

   Returns: If value is found in the invocation list for this instance,then a new System.Delegate without 

    value in its invocation list; otherwise,this instance with its original invocation list.
  """
  pass
 def __init__(self,*args):
  """ x.__init__(...) initializes x; see x.__class__.__doc__ for signaturex.__init__(...) initializes x; see x.__class__.__doc__ for signaturex.__init__(...) initializes x; see x.__class__.__doc__ for signature """
  pass
 @staticmethod
 def __new__(self,object,method):
  """ __new__(cls: type,object: object,method: IntPtr) """
  pass
 def __reduce_ex__(self,*args):
  pass

class ToolboxComponentsCreatedEventArgs(EventArgs):
 """
 Provides data for the System.Drawing.Design.ToolboxItem.ComponentsCreated event that occurs when components are added to the toolbox.

 

 ToolboxComponentsCreatedEventArgs(components: Array[IComponent])
 """
 @staticmethod
 def __new__(self,components):
  """ __new__(cls: type,components: Array[IComponent]) """
  pass
 Components=property(lambda self: object(),lambda self,v: None,lambda self: None)
 """Gets or sets an array containing the components to add to the toolbox.



Get: Components(self: ToolboxComponentsCreatedEventArgs) -> Array[IComponent]



"""



class ToolboxComponentsCreatedEventHandler(MulticastDelegate,ICloneable,ISerializable):
 """
 Represents the method that handles the System.Drawing.Design.ToolboxItem.ComponentsCreated event.

 

 ToolboxComponentsCreatedEventHandler(object: object,method: IntPtr)
 """
 def BeginInvoke(self,sender,e,callback,object):
  """ BeginInvoke(self: ToolboxComponentsCreatedEventHandler,sender: object,e: ToolboxComponentsCreatedEventArgs,callback: AsyncCallback,object: object) -> IAsyncResult """
  pass
 def CombineImpl(self,*args):
  """
  CombineImpl(self: MulticastDelegate,follow: Delegate) -> Delegate

  

   Combines this System.Delegate with the specified System.Delegate to form a new delegate.

  

   follow: The delegate to combine with this delegate.

   Returns: A delegate that is the new root of the System.MulticastDelegate invocation list.
  """
  pass
 def DynamicInvokeImpl(self,*args):
  """
  DynamicInvokeImpl(self: Delegate,args: Array[object]) -> object

  

   Dynamically invokes (late-bound) the method represented by the current delegate.

  

   args: An array of objects that are the arguments to pass to the method represented by the current 

    delegate.-or- null,if the method represented by the current delegate does not require 

    arguments.

  

   Returns: The object returned by the method represented by the delegate.
  """
  pass
 def EndInvoke(self,result):
  """ EndInvoke(self: ToolboxComponentsCreatedEventHandler,result: IAsyncResult) """
  pass
 def GetMethodImpl(self,*args):
  """
  GetMethodImpl(self: MulticastDelegate) -> MethodInfo

  

   Returns a static method represented by the current System.MulticastDelegate.

   Returns: A static method represented by the current System.MulticastDelegate.
  """
  pass
 def Invoke(self,sender,e):
  """ Invoke(self: ToolboxComponentsCreatedEventHandler,sender: object,e: ToolboxComponentsCreatedEventArgs) """
  pass
 def RemoveImpl(self,*args):
  """
  RemoveImpl(self: MulticastDelegate,value: Delegate) -> Delegate

  

   Removes an element from the invocation list of this System.MulticastDelegate that is equal to 

    the specified delegate.

  

  

   value: The delegate to search for in the invocation list.

   Returns: If value is found in the invocation list for this instance,then a new System.Delegate without 

    value in its invocation list; otherwise,this instance with its original invocation list.
  """
  pass
 def __init__(self,*args):
  """ x.__init__(...) initializes x; see x.__class__.__doc__ for signaturex.__init__(...) initializes x; see x.__class__.__doc__ for signaturex.__init__(...) initializes x; see x.__class__.__doc__ for signature """
  pass
 @staticmethod
 def __new__(self,object,method):
  """ __new__(cls: type,object: object,method: IntPtr) """
  pass
 def __reduce_ex__(self,*args):
  pass

class ToolboxComponentsCreatingEventArgs(EventArgs):
 """
 Provides data for the System.Drawing.Design.ToolboxItem.ComponentsCreating event that occurs when components are added to the toolbox.

 

 ToolboxComponentsCreatingEventArgs(host: IDesignerHost)
 """
 @staticmethod
 def __new__(self,host):
  """ __new__(cls: type,host: IDesignerHost) """
  pass
 DesignerHost=property(lambda self: object(),lambda self,v: None,lambda self: None)
 """Gets or sets an instance of the System.ComponentModel.Design.IDesignerHost that made the request to create toolbox components.



Get: DesignerHost(self: ToolboxComponentsCreatingEventArgs) -> IDesignerHost



"""



class ToolboxComponentsCreatingEventHandler(MulticastDelegate,ICloneable,ISerializable):
 """
 Represents the method that handles the System.Drawing.Design.ToolboxItem.ComponentsCreating event.

 

 ToolboxComponentsCreatingEventHandler(object: object,method: IntPtr)
 """
 def BeginInvoke(self,sender,e,callback,object):
  """ BeginInvoke(self: ToolboxComponentsCreatingEventHandler,sender: object,e: ToolboxComponentsCreatingEventArgs,callback: AsyncCallback,object: object) -> IAsyncResult """
  pass
 def CombineImpl(self,*args):
  """
  CombineImpl(self: MulticastDelegate,follow: Delegate) -> Delegate

  

   Combines this System.Delegate with the specified System.Delegate to form a new delegate.

  

   follow: The delegate to combine with this delegate.

   Returns: A delegate that is the new root of the System.MulticastDelegate invocation list.
  """
  pass
 def DynamicInvokeImpl(self,*args):
  """
  DynamicInvokeImpl(self: Delegate,args: Array[object]) -> object

  

   Dynamically invokes (late-bound) the method represented by the current delegate.

  

   args: An array of objects that are the arguments to pass to the method represented by the current 

    delegate.-or- null,if the method represented by the current delegate does not require 

    arguments.

  

   Returns: The object returned by the method represented by the delegate.
  """
  pass
 def EndInvoke(self,result):
  """ EndInvoke(self: ToolboxComponentsCreatingEventHandler,result: IAsyncResult) """
  pass
 def GetMethodImpl(self,*args):
  """
  GetMethodImpl(self: MulticastDelegate) -> MethodInfo

  

   Returns a static method represented by the current System.MulticastDelegate.

   Returns: A static method represented by the current System.MulticastDelegate.
  """
  pass
 def Invoke(self,sender,e):
  """ Invoke(self: ToolboxComponentsCreatingEventHandler,sender: object,e: ToolboxComponentsCreatingEventArgs) """
  pass
 def RemoveImpl(self,*args):
  """
  RemoveImpl(self: MulticastDelegate,value: Delegate) -> Delegate

  

   Removes an element from the invocation list of this System.MulticastDelegate that is equal to 

    the specified delegate.

  

  

   value: The delegate to search for in the invocation list.

   Returns: If value is found in the invocation list for this instance,then a new System.Delegate without 

    value in its invocation list; otherwise,this instance with its original invocation list.
  """
  pass
 def __init__(self,*args):
  """ x.__init__(...) initializes x; see x.__class__.__doc__ for signaturex.__init__(...) initializes x; see x.__class__.__doc__ for signaturex.__init__(...) initializes x; see x.__class__.__doc__ for signature """
  pass
 @staticmethod
 def __new__(self,object,method):
  """ __new__(cls: type,object: object,method: IntPtr) """
  pass
 def __reduce_ex__(self,*args):
  pass

class ToolboxItem(object,ISerializable):
 """
 Provides a base implementation of a toolbox item.

 

 ToolboxItem()

 ToolboxItem(toolType: Type)
 """
 def CheckUnlocked(self,*args):
  """
  CheckUnlocked(self: ToolboxItem)

   Throws an exception if the toolbox item is currently locked.
  """
  pass
 def CreateComponents(self,host=None,defaultValues=None):
  """
  CreateComponents(self: ToolboxItem,host: IDesignerHost,defaultValues: IDictionary) -> Array[IComponent]

  

   Creates the components that the toolbox item is configured to create,using the specified 

    designer host and default values.

  

  

   host: The System.ComponentModel.Design.IDesignerHost to use when creating the components.

   defaultValues: A dictionary of property name/value pairs of default values with which to initialize the 

    component.

  

   Returns: An array of created System.ComponentModel.IComponent objects.

  CreateComponents(self: ToolboxItem,host: IDesignerHost) -> Array[IComponent]

  

   Creates the components that the toolbox item is configured to create,using the specified 

    designer host.

  

  

   host: The System.ComponentModel.Design.IDesignerHost to use when creating the components.

   Returns: An array of created System.ComponentModel.IComponent objects.

  CreateComponents(self: ToolboxItem) -> Array[IComponent]

  

   Creates the components that the toolbox item is configured to create.

   Returns: An array of created System.ComponentModel.IComponent objects.
  """
  pass
 def CreateComponentsCore(self,*args):
  """
  CreateComponentsCore(self: ToolboxItem,host: IDesignerHost,defaultValues: IDictionary) -> Array[IComponent]

  

   Creates an array of components when the toolbox item is invoked.

  

   host: The designer host to use when creating components.

   defaultValues: A dictionary of property name/value pairs of default values with which to initialize the 

    component.

  

   Returns: An array of created System.ComponentModel.IComponent objects.

  CreateComponentsCore(self: ToolboxItem,host: IDesignerHost) -> Array[IComponent]

  

   Creates a component or an array of components when the toolbox item is invoked.

  

   host: The System.ComponentModel.Design.IDesignerHost to host the toolbox item.

   Returns: An array of created System.ComponentModel.IComponent objects.
  """
  pass
 def Deserialize(self,*args):
  """
  Deserialize(self: ToolboxItem,info: SerializationInfo,context: StreamingContext)

   Loads the state of the toolbox item from the specified serialization information object.

  

   info: The System.Runtime.Serialization.SerializationInfo to load from.

   context: A System.Runtime.Serialization.StreamingContext that indicates the stream characteristics.
  """
  pass
 def Equals(self,obj):
  """
  Equals(self: ToolboxItem,obj: object) -> bool

  

   Determines whether two System.Drawing.Design.ToolboxItem instances are equal.

  

   obj: The System.Drawing.Design.ToolboxItem to compare with the current 

    System.Drawing.Design.ToolboxItem.

  

   Returns: true if the specified System.Drawing.Design.ToolboxItem is equal to the current 

    System.Drawing.Design.ToolboxItem; otherwise,false.
  """
  pass
 def FilterPropertyValue(self,*args):
  """
  FilterPropertyValue(self: ToolboxItem,propertyName: str,value: object) -> object

  

   Filters a property value before returning it.

  

   propertyName: The name of the property to filter.

   value: The value against which to filter the property.

   Returns: A filtered property value.
  """
  pass
 def GetHashCode(self):
  """
  GetHashCode(self: ToolboxItem) -> int

  

   Returns the hash code for this instance.

   Returns: A hash code for the current System.Drawing.Design.ToolboxItem.
  """
  pass
 def GetType(self,host=None):
  """
  GetType(self: ToolboxItem,host: IDesignerHost) -> Type

  

   Enables access to the type associated with the toolbox item.

  

   host: The designer host to query for System.ComponentModel.Design.ITypeResolutionService.

   Returns: The type associated with the toolbox item.
  """
  pass
 def Initialize(self,type):
  """
  Initialize(self: ToolboxItem,type: Type)

   Initializes the current toolbox item with the specified type to create.

  

   type: The System.Type that the toolbox item creates.
  """
  pass
 def Lock(self):
  """
  Lock(self: ToolboxItem)

   Locks the toolbox item and prevents changes to its properties.
  """
  pass
 def OnComponentsCreated(self,*args):
  """
  OnComponentsCreated(self: ToolboxItem,args: ToolboxComponentsCreatedEventArgs)

   Raises the System.Drawing.Design.ToolboxItem.ComponentsCreated event.

  

   args: A System.Drawing.Design.ToolboxComponentsCreatedEventArgs that provides data for the event.
  """
  pass
 def OnComponentsCreating(self,*args):
  """
  OnComponentsCreating(self: ToolboxItem,args: ToolboxComponentsCreatingEventArgs)

   Raises the System.Drawing.Design.ToolboxItem.ComponentsCreating event.

  

   args: A System.Drawing.Design.ToolboxComponentsCreatingEventArgs that provides data for the event.
  """
  pass
 def Serialize(self,*args):
  """
  Serialize(self: ToolboxItem,info: SerializationInfo,context: StreamingContext)

   Saves the state of the toolbox item to the specified serialization information object.

  

   info: The System.Runtime.Serialization.SerializationInfo to save to.

   context: A System.Runtime.Serialization.StreamingContext that indicates the stream characteristics.
  """
  pass
 def ToString(self):
  """
  ToString(self: ToolboxItem) -> str

  

   Returns a System.String that represents the current System.Drawing.Design.ToolboxItem.

   Returns: A System.String that represents the current System.Drawing.Design.ToolboxItem.
  """
  pass
 def ValidatePropertyType(self,*args):
  """
  ValidatePropertyType(self: ToolboxItem,propertyName: str,value: object,expectedType: Type,allowNull: bool)

   Validates that an object is of a given type.

  

   propertyName: The name of the property to validate.

   value: Optional value against which to validate.

   expectedType: The expected type of the property.

   allowNull: true to allow null; otherwise,false.
  """
  pass
 def ValidatePropertyValue(self,*args):
  """
  ValidatePropertyValue(self: ToolboxItem,propertyName: str,value: object) -> object

  

   Validates a property before it is assigned to the property dictionary.

  

   propertyName: The name of the property to validate.

   value: The value against which to validate.

   Returns: The value used to perform validation.
  """
  pass
 def __eq__(self,*args):
  """ x.__eq__(y) <==> x==y """
  pass
 def __init__(self,*args):
  """ x.__init__(...) initializes x; see x.__class__.__doc__ for signaturex.__init__(...) initializes x; see x.__class__.__doc__ for signaturex.__init__(...) initializes x; see x.__class__.__doc__ for signature """
  pass
 @staticmethod
 def __new__(self,toolType=None):
  """
  __new__(cls: type)

  __new__(cls: type,toolType: Type)
  """
  pass
 def __ne__(self,*args):
  pass
 def __reduce_ex__(self,*args):
  pass
 def __repr__(self,*args):
  """ __repr__(self: object) -> str """
  pass
 def __str__(self,*args):
  pass
 AssemblyName=property(lambda self: object(),lambda self,v: None,lambda self: None)
 """Gets or sets the name of the assembly that contains the type or types that the toolbox item creates.



Get: AssemblyName(self: ToolboxItem) -> AssemblyName



Set: AssemblyName(self: ToolboxItem)=value

"""

 Bitmap=property(lambda self: object(),lambda self,v: None,lambda self: None)
 """Gets or sets a bitmap to represent the toolbox item in the toolbox.



Get: Bitmap(self: ToolboxItem) -> Bitmap



Set: Bitmap(self: ToolboxItem)=value

"""

 Company=property(lambda self: object(),lambda self,v: None,lambda self: None)
 """Gets or sets the company name for this System.Drawing.Design.ToolboxItem.



Get: Company(self: ToolboxItem) -> str



Set: Company(self: ToolboxItem)=value

"""

 ComponentType=property(lambda self: object(),lambda self,v: None,lambda self: None)
 """Gets the component type for this System.Drawing.Design.ToolboxItem.



Get: ComponentType(self: ToolboxItem) -> str



"""

 DependentAssemblies=property(lambda self: object(),lambda self,v: None,lambda self: None)
 """Gets or sets the System.Reflection.AssemblyName for the toolbox item.



Get: DependentAssemblies(self: ToolboxItem) -> Array[AssemblyName]



Set: DependentAssemblies(self: ToolboxItem)=value

"""

 Description=property(lambda self: object(),lambda self,v: None,lambda self: None)
 """Gets or sets the description for this System.Drawing.Design.ToolboxItem.



Get: Description(self: ToolboxItem) -> str



Set: Description(self: ToolboxItem)=value

"""

 DisplayName=property(lambda self: object(),lambda self,v: None,lambda self: None)
 """Gets or sets the display name for the toolbox item.



Get: DisplayName(self: ToolboxItem) -> str



Set: DisplayName(self: ToolboxItem)=value

"""

 Filter=property(lambda self: object(),lambda self,v: None,lambda self: None)
 """Gets or sets the filter that determines whether the toolbox item can be used on a destination component.



Get: Filter(self: ToolboxItem) -> ICollection



Set: Filter(self: ToolboxItem)=value

"""

 IsTransient=property(lambda self: object(),lambda self,v: None,lambda self: None)
 """Gets a value indicating whether the toolbox item is transient.



Get: IsTransient(self: ToolboxItem) -> bool



Set: IsTransient(self: ToolboxItem)=value

"""

 Locked=property(lambda self: object(),lambda self,v: None,lambda self: None)
 """Gets a value indicating whether the System.Drawing.Design.ToolboxItem is currently locked.



Get: Locked(self: ToolboxItem) -> bool



"""

 OriginalBitmap=property(lambda self: object(),lambda self,v: None,lambda self: None)
 """Get: OriginalBitmap(self: ToolboxItem) -> Bitmap



Set: OriginalBitmap(self: ToolboxItem)=value

"""

 Properties=property(lambda self: object(),lambda self,v: None,lambda self: None)
 """Gets a dictionary of properties.



Get: Properties(self: ToolboxItem) -> IDictionary



"""

 TypeName=property(lambda self: object(),lambda self,v: None,lambda self: None)
 """Gets or sets the fully qualified name of the type of System.ComponentModel.IComponent that the toolbox item creates when invoked.



Get: TypeName(self: ToolboxItem) -> str



Set: TypeName(self: ToolboxItem)=value

"""

 Version=property(lambda self: object(),lambda self,v: None,lambda self: None)
 """Gets the version for this System.Drawing.Design.ToolboxItem.



Get: Version(self: ToolboxItem) -> str



"""


 ComponentsCreated=None
 ComponentsCreating=None


class ToolboxItemCollection(ReadOnlyCollectionBase,ICollection,IEnumerable):
 """
 Represents a collection of toolbox items.

 

 ToolboxItemCollection(value: ToolboxItemCollection)

 ToolboxItemCollection(value: Array[ToolboxItem])
 """
 def Contains(self,value):
  """
  Contains(self: ToolboxItemCollection,value: ToolboxItem) -> bool

  

   Indicates whether the collection contains the specified System.Drawing.Design.ToolboxItem.

  

   value: A System.Drawing.Design.ToolboxItem to search the collection for.

   Returns: true if the collection contains the specified object; otherwise,false.
  """
  pass
 def CopyTo(self,array,index):
  """
  CopyTo(self: ToolboxItemCollection,array: Array[ToolboxItem],index: int)

   Copies the collection to the specified array beginning with the specified destination index.

  

   array: The array to copy to.

   index: The index to begin copying to.
  """
  pass
 def IndexOf(self,value):
  """
  IndexOf(self: ToolboxItemCollection,value: ToolboxItem) -> int

  

   Gets the index of the specified System.Drawing.Design.ToolboxItem,if it exists in the 

    collection.

  

  

   value: A System.Drawing.Design.ToolboxItem to get the index of in the collection.

   Returns: The index of the specified System.Drawing.Design.ToolboxItem.
  """
  pass
 def __getitem__(self,*args):
  """ x.__getitem__(y) <==> x[y] """
  pass
 def __init__(self,*args):
  """ x.__init__(...) initializes x; see x.__class__.__doc__ for signaturex.__init__(...) initializes x; see x.__class__.__doc__ for signaturex.__init__(...) initializes x; see x.__class__.__doc__ for signature """
  pass
 def __iter__(self,*args):
  """ __iter__(self: IEnumerable) -> object """
  pass
 @staticmethod
 def __new__(self,value):
  """
  __new__(cls: type,value: ToolboxItemCollection)

  __new__(cls: type,value: Array[ToolboxItem])
  """
  pass
 InnerList=property(lambda self: object(),lambda self,v: None,lambda self: None)
 """Gets the list of elements contained in the System.Collections.ReadOnlyCollectionBase instance.



"""



class ToolboxItemCreatorCallback(MulticastDelegate,ICloneable,ISerializable):
 """
 Provides a callback mechanism that can create a System.Drawing.Design.ToolboxItem.

 

 ToolboxItemCreatorCallback(object: object,method: IntPtr)
 """
 def BeginInvoke(self,serializedObject,format,callback,object):
  """ BeginInvoke(self: ToolboxItemCreatorCallback,serializedObject: object,format: str,callback: AsyncCallback,object: object) -> IAsyncResult """
  pass
 def CombineImpl(self,*args):
  """
  CombineImpl(self: MulticastDelegate,follow: Delegate) -> Delegate

  

   Combines this System.Delegate with the specified System.Delegate to form a new delegate.

  

   follow: The delegate to combine with this delegate.

   Returns: A delegate that is the new root of the System.MulticastDelegate invocation list.
  """
  pass
 def DynamicInvokeImpl(self,*args):
  """
  DynamicInvokeImpl(self: Delegate,args: Array[object]) -> object

  

   Dynamically invokes (late-bound) the method represented by the current delegate.

  

   args: An array of objects that are the arguments to pass to the method represented by the current 

    delegate.-or- null,if the method represented by the current delegate does not require 

    arguments.

  

   Returns: The object returned by the method represented by the delegate.
  """
  pass
 def EndInvoke(self,result):
  """ EndInvoke(self: ToolboxItemCreatorCallback,result: IAsyncResult) -> ToolboxItem """
  pass
 def GetMethodImpl(self,*args):
  """
  GetMethodImpl(self: MulticastDelegate) -> MethodInfo

  

   Returns a static method represented by the current System.MulticastDelegate.

   Returns: A static method represented by the current System.MulticastDelegate.
  """
  pass
 def Invoke(self,serializedObject,format):
  """ Invoke(self: ToolboxItemCreatorCallback,serializedObject: object,format: str) -> ToolboxItem """
  pass
 def RemoveImpl(self,*args):
  """
  RemoveImpl(self: MulticastDelegate,value: Delegate) -> Delegate

  

   Removes an element from the invocation list of this System.MulticastDelegate that is equal to 

    the specified delegate.

  

  

   value: The delegate to search for in the invocation list.

   Returns: If value is found in the invocation list for this instance,then a new System.Delegate without 

    value in its invocation list; otherwise,this instance with its original invocation list.
  """
  pass
 def __init__(self,*args):
  """ x.__init__(...) initializes x; see x.__class__.__doc__ for signaturex.__init__(...) initializes x; see x.__class__.__doc__ for signaturex.__init__(...) initializes x; see x.__class__.__doc__ for signature """
  pass
 @staticmethod
 def __new__(self,object,method):
  """ __new__(cls: type,object: object,method: IntPtr) """
  pass
 def __reduce_ex__(self,*args):
  pass

class UITypeEditor(object):
 """
 Provides a base class that can be used to design value editors that can provide a user interface (UI) for representing and editing the values of objects of the supported data types.

 

 UITypeEditor()
 """
 def EditValue(self,*__args):
  """
  EditValue(self: UITypeEditor,context: ITypeDescriptorContext,provider: IServiceProvider,value: object) -> object

  

   Edits the specified object's value using the editor style indicated by the 

    System.Drawing.Design.UITypeEditor.GetEditStyle method.

  

  

   context: An System.ComponentModel.ITypeDescriptorContext that can be used to gain additional context 

    information.

  

   provider: An System.IServiceProvider that this editor can use to obtain services.

   value: The object to edit.

   Returns: The new value of the object. If the value of the object has not changed,this should return the 

    same object it was passed.

  

  EditValue(self: UITypeEditor,provider: IServiceProvider,value: object) -> object

  

   Edits the value of the specified object using the editor style indicated by the 

    System.Drawing.Design.UITypeEditor.GetEditStyle method.

  

  

   provider: An System.IServiceProvider that this editor can use to obtain services.

   value: The object to edit.

   Returns: The new value of the object.
  """
  pass
 def GetEditStyle(self,context=None):
  """
  GetEditStyle(self: UITypeEditor,context: ITypeDescriptorContext) -> UITypeEditorEditStyle

  

   Gets the editor style used by the 

    System.Drawing.Design.UITypeEditor.EditValue(System.IServiceProvider,System.Object) method.

  

  

   context: An System.ComponentModel.ITypeDescriptorContext that can be used to gain additional context 

    information.

  

   Returns: A System.Drawing.Design.UITypeEditorEditStyle value that indicates the style of editor used by 

    the System.Drawing.Design.UITypeEditor.EditValue(System.IServiceProvider,System.Object) method. 

    If the System.Drawing.Design.UITypeEditor does not support this method,then 

    System.Drawing.Design.UITypeEditor.GetEditStyle will return 

    System.Drawing.Design.UITypeEditorEditStyle.None.

  

  GetEditStyle(self: UITypeEditor) -> UITypeEditorEditStyle

  

   Gets the editor style used by the 

    System.Drawing.Design.UITypeEditor.EditValue(System.IServiceProvider,System.Object) method.

  

   Returns: A System.Drawing.Design.UITypeEditorEditStyle enumeration value that indicates the style of 

    editor used by the current System.Drawing.Design.UITypeEditor. By default,this method will 

    return System.Drawing.Design.UITypeEditorEditStyle.None.
  """
  pass
 def GetPaintValueSupported(self,context=None):
  """
  GetPaintValueSupported(self: UITypeEditor,context: ITypeDescriptorContext) -> bool

  

   Indicates whether the specified context supports painting a representation of an object's value 

    within the specified context.

  

  

   context: An System.ComponentModel.ITypeDescriptorContext that can be used to gain additional context 

    information.

  

   Returns: true if 

    System.Drawing.Design.UITypeEditor.PaintValue(System.Object,System.Drawing.Graphics,System.Drawin

    g.Rectangle) is implemented; otherwise,false.

  

  GetPaintValueSupported(self: UITypeEditor) -> bool

  

   Indicates whether this editor supports painting a representation of an object's value.

   Returns: true if 

    System.Drawing.Design.UITypeEditor.PaintValue(System.Object,System.Drawing.Graphics,System.Drawin

    g.Rectangle) is implemented; otherwise,false.
  """
  pass
 def PaintValue(self,*__args):
  """
  PaintValue(self: UITypeEditor,e: PaintValueEventArgs)

   Paints a representation of the value of an object using the specified 

    System.Drawing.Design.PaintValueEventArgs.

  

  

   e: A System.Drawing.Design.PaintValueEventArgs that indicates what to paint and where to paint it.

  PaintValue(self: UITypeEditor,value: object,canvas: Graphics,rectangle: Rectangle)

   Paints a representation of the value of the specified object to the specified canvas.

  

   value: The object whose value this type editor will display.

   canvas: A drawing canvas on which to paint the representation of the object's value.

   rectangle: A System.Drawing.Rectangle within whose boundaries to paint the value.
  """
  pass
 IsDropDownResizable=property(lambda self: object(),lambda self,v: None,lambda self: None)
 """Gets a value indicating whether drop-down editors should be resizable by the user.



Get: IsDropDownResizable(self: UITypeEditor) -> bool



"""



class UITypeEditorEditStyle(Enum,IComparable,IFormattable,IConvertible):
 """
 Specifies identifiers that indicate the value editing style of a System.Drawing.Design.UITypeEditor.

 

 enum UITypeEditorEditStyle,values: DropDown (3),Modal (2),None (1)
 """
 def __eq__(self,*args):
  """ x.__eq__(y) <==> x==yx.__eq__(y) <==> x==yx.__eq__(y) <==> x==y """
  pass
 def __format__(self,*args):
  """ __format__(formattable: IFormattable,format: str) -> str """
  pass
 def __ge__(self,*args):
  pass
 def __gt__(self,*args):
  pass
 def __init__(self,*args):
  """ x.__init__(...) initializes x; see x.__class__.__doc__ for signaturex.__init__(...) initializes x; see x.__class__.__doc__ for signaturex.__init__(...) initializes x; see x.__class__.__doc__ for signature """
  pass
 def __le__(self,*args):
  pass
 def __lt__(self,*args):
  pass
 def __ne__(self,*args):
  pass
 def __reduce_ex__(self,*args):
  pass
 def __str__(self,*args):
  pass
 DropDown=None
 Modal=None
 None=None
 value__=None





class HandledEventArgs(EventArgs):
 """
 Provides data for events that can be handled completely in an event handler.

 

 HandledEventArgs()

 HandledEventArgs(defaultHandledValue: bool)
 """
 @staticmethod
 def __new__(self,defaultHandledValue=None):
  """
  __new__(cls: type)

  __new__(cls: type,defaultHandledValue: bool)
  """
  pass
 Handled=property(lambda self: object(),lambda self,v: None,lambda self: None)
 """Gets or sets a value that indicates whether the event handler has completely handled the event or whether the system should continue its own processing.



Get: Handled(self: HandledEventArgs) -> bool



Set: Handled(self: HandledEventArgs)=value

"""





class StringComparer(object,IComparer,IEqualityComparer,IComparer[str],IEqualityComparer[str]):
 """ Represents a string comparison operation that uses specific case and culture-based or ordinal comparison rules. """
 def Compare(self,x,y):
  """
  Compare(self: StringComparer,x: str,y: str) -> int

  

   When overridden in a derived class,compares two strings and returns an indication of their 

    relative sort order.

  

  

   x: A string to compare to y.

   y: A string to compare to x.

   Returns: A signed integer that indicates the relative values of x and y,as shown in the following 

    table.ValueMeaningLess than zerox is less than y.-or-x is null.Zerox is equal to y.Greater than 

    zerox is greater than y.-or-y is null.

  

  Compare(self: StringComparer,x: object,y: object) -> int

  

   When overridden in a derived class,compares two objects and returns an indication of their 

    relative sort order.

  

  

   x: An object to compare to y.

   y: An object to compare to x.

   Returns: A signed integer that indicates the relative values of x and y,as shown in the following 

    table.ValueMeaningLess than zerox is less than y. -or-x is null.Zerox is equal to y.Greater than 

    zerox is greater than y.-or-y is null.
  """
  pass
 @staticmethod
 def Create(culture,ignoreCase):
  """
  Create(culture: CultureInfo,ignoreCase: bool) -> StringComparer

  

   Creates a System.StringComparer object that compares strings according to the rules of a 

    specified culture.

  

  

   culture: A culture whose linguistic rules are used to perform a string comparison.

   ignoreCase: true to specify that comparison operations be case-insensitive; false to specify that comparison 

    operations be case-sensitive.

  

   Returns: A new System.StringComparer object that performs string comparisons according to the comparison 

    rules used by the culture parameter and the case rule specified by the ignoreCase parameter.
  """
  pass
 def Equals(self,*__args):
  """
  Equals(self: StringComparer,x: str,y: str) -> bool

  

   When overridden in a derived class,indicates whether two strings are equal.

  

   x: A string to compare to y.

   y: A string to compare to x.

   Returns: true if x and y refer to the same object,or x and y are equal; otherwise,false.

  Equals(self: StringComparer,x: object,y: object) -> bool

  

   When overridden in a derived class,indicates whether two objects are equal.

  

   x: An object to compare to y.

   y: An object to compare to x.

   Returns: true if x and y refer to the same object,or x and y are both the same type of object and those 

    objects are equal; otherwise,false.
  """
  pass
 def GetHashCode(self,obj=None):
  """
  GetHashCode(self: StringComparer,obj: str) -> int

  

   When overridden in a derived class,gets the hash code for the specified string.

  

   obj: A string.

   Returns: A 32-bit signed hash code calculated from the value of the obj parameter.

  GetHashCode(self: StringComparer,obj: object) -> int

  

   When overridden in a derived class,gets the hash code for the specified object.

  

   obj: An object.

   Returns: A 32-bit signed hash code calculated from the value of the obj parameter.
  """
  pass
 def __cmp__(self,*args):
  """ x.__cmp__(y) <==> cmp(x,y)x.__cmp__(y) <==> cmp(x,y) """
  pass
 def __eq__(self,*args):
  """ x.__eq__(y) <==> x==yx.__eq__(y) <==> x==y """
  pass
 def __init__(self,*args):
  """ x.__init__(...) initializes x; see x.__class__.__doc__ for signaturex.__init__(...) initializes x; see x.__class__.__doc__ for signaturex.__init__(...) initializes x; see x.__class__.__doc__ for signature """
  pass
 def __reduce_ex__(self,*args):
  pass
 def __repr__(self,*args):
  """ __repr__(self: object) -> str """
  pass
 CurrentCulture=None
 CurrentCultureIgnoreCase=None
 InvariantCulture=None
 InvariantCultureIgnoreCase=None
 Ordinal=None
 OrdinalIgnoreCase=None




class Enum(object,IComparable,IFormattable,IConvertible):
 """ Provides the base class for enumerations. """
 def CompareTo(self,target):
  """
  CompareTo(self: Enum,target: object) -> int

  

   Compares this instance to a specified object and returns an indication of their relative values.

  

   target: An object to compare,or null.

   Returns: A signed number that indicates the relative values of this instance and target.Value Meaning 

    Less than zero The value of this instance is less than the value of target. Zero The value of 

    this instance is equal to the value of target. Greater than zero The value of this instance is 

    greater than the value of target.-or- target is null.
  """
  pass
 def Equals(self,obj):
  """
  Equals(self: Enum,obj: object) -> bool

  

   Returns a value indicating whether this instance is equal to a specified object.

  

   obj: An object to compare with this instance,or null.

   Returns: true if obj is an System.Enum with the same underlying type and value as this instance; 

    otherwise,false.
  """
  pass
 @staticmethod
 def Format(enumType,value,format):
  """
  Format(enumType: Type,value: object,format: str) -> str

  

   Converts the specified value of a specified enumerated type to its equivalent string 

    representation according to the specified format.

  

  

   enumType: The enumeration type of the value to convert.

   value: The value to convert.

   format: The output format to use.

   Returns: A string representation of value.
  """
  pass
 def GetHashCode(self):
  """
  GetHashCode(self: Enum) -> int

  

   Returns the hash code for the value of this instance.

   Returns: A 32-bit signed integer hash code.
  """
  pass
 @staticmethod
 def GetName(enumType,value):
  """
  GetName(enumType: Type,value: object) -> str

  

   Retrieves the name of the constant in the specified enumeration that has the specified value.

  

   enumType: An enumeration type.

   value: The value of a particular enumerated constant in terms of its underlying type.

   Returns: A string containing the name of the enumerated constant in enumType whose value is value; or 

    null if no such constant is found.
  """
  pass
 @staticmethod
 def GetNames(enumType):
  """
  GetNames(enumType: Type) -> Array[str]

  

   Retrieves an array of the names of the constants in a specified enumeration.

  

   enumType: An enumeration type.

   Returns: A string array of the names of the constants in enumType.
  """
  pass
 def GetTypeCode(self):
  """
  GetTypeCode(self: Enum) -> TypeCode

  

   Returns the underlying System.TypeCode for this instance.

   Returns: The type for this instance.
  """
  pass
 @staticmethod
 def GetUnderlyingType(enumType):
  """
  GetUnderlyingType(enumType: Type) -> Type

  

   Returns the underlying type of the specified enumeration.

  

   enumType: The enumeration whose underlying type will be retrieved.

   Returns: The underlying type of enumType.
  """
  pass
 @staticmethod
 def GetValues(enumType):
  """
  GetValues(enumType: Type) -> Array

  

   Retrieves an array of the values of the constants in a specified enumeration.

  

   enumType: The enumeration type to retrieve values from.

   Returns: An array that contains the values of the constants in enumType.
  """
  pass
 def HasFlag(self,flag):
  """
  HasFlag(self: Enum,flag: Enum) -> bool

  

   Determines whether one or more bit fields are set in the current instance.

  

   flag: An enumeration value.

   Returns: true if the bit field or bit fields that are set in flag are also set in the current instance; 

    otherwise,false.
  """
  pass
 @staticmethod
 def IsDefined(enumType,value):
  """
  IsDefined(enumType: Type,value: object) -> bool

  

   Indicates whether a constant with a specified value exists in a specified enumeration.

  

   enumType: The enumeration type to check.

   value: The value or name of a constant in enumType.

   Returns: true if a constant in enumType has a value equal to value; otherwise,false.
  """
  pass
 @staticmethod
 def Parse(enumType,value,ignoreCase=None):
  """
  Parse(enumType: Type,value: str,ignoreCase: bool) -> object

  

   Converts the string representation of the name or numeric value of one or more enumerated 

    constants to an equivalent enumerated object. A parameter specifies whether the operation is 

    case-insensitive.

  

  

   enumType: An enumeration type.

   value: A string containing the name or value to convert.

   ignoreCase: true to ignore case; false to regard case.

   Returns: An object of type enumType whose value is represented by value.

  Parse(enumType: Type,value: str) -> object

  

   Converts the string representation of the name or numeric value of one or more enumerated 

    constants to an equivalent enumerated object.

  

  

   enumType: An enumeration type.

   value: A string containing the name or value to convert.

   Returns: An object of type enumType whose value is represented by value.
  """
  pass
 @staticmethod
 def ToObject(enumType,value):
  """
  ToObject(enumType: Type,value: UInt32) -> object

  

   Converts the specified 32-bit unsigned integer value to an enumeration member.

  

   enumType: The enumeration type to return.

   value: The value to convert to an enumeration member.

   Returns: An instance of the enumeration set to value.

  ToObject(enumType: Type,value: UInt16) -> object

  

   Converts the specified 16-bit unsigned integer value to an enumeration member.

  

   enumType: The enumeration type to return.

   value: The value to convert to an enumeration member.

   Returns: An instance of the enumeration set to value.

  ToObject(enumType: Type,value: UInt64) -> object

  

   Converts the specified 64-bit unsigned integer value to an enumeration member.

  

   enumType: The enumeration type to return.

   value: The value to convert to an enumeration member.

   Returns: An instance of the enumeration set to value.

  ToObject(enumType: Type,value: Int64) -> object

  

   Converts the specified 64-bit signed integer to an enumeration member.

  

   enumType: The enumeration type to return.

   value: The value to convert to an enumeration member.

   Returns: An instance of the enumeration set to value.

  ToObject(enumType: Type,value: Byte) -> object

  

   Converts the specified 8-bit unsigned integer to an enumeration member.

  

   enumType: The enumeration type to return.

   value: The value to convert to an enumeration member.

   Returns: An instance of the enumeration set to value.

  ToObject(enumType: Type,value: SByte) -> object

  

   Converts the specified 8-bit signed integer value to an enumeration member.

  

   enumType: The enumeration type to return.

   value: The value to convert to an enumeration member.

   Returns: An instance of the enumeration set to value.

  ToObject(enumType: Type,value: object) -> object

  

   Converts the specified object with an integer value to an enumeration member.

  

   enumType: The enumeration type to return.

   value: The value convert to an enumeration member.

   Returns: An enumeration object whose value is value.

  ToObject(enumType: Type,value: int) -> object

  

   Converts the specified 32-bit signed integer to an enumeration member.

  

   enumType: The enumeration type to return.

   value: The value to convert to an enumeration member.

   Returns: An instance of the enumeration set to value.

  ToObject(enumType: Type,value: Int16) -> object

  

   Converts the specified 16-bit signed integer to an enumeration member.

  

   enumType: The enumeration type to return.

   value: The value to convert to an enumeration member.

   Returns: An instance of the enumeration set to value.
  """
  pass
 def ToString(self,*__args):
  """
  ToString(self: Enum,format: str) -> str

  

   Converts the value of this instance to its equivalent string representation using the specified 

    format.

  

  

   format: A format string.

   Returns: The string representation of the value of this instance as specified by format.

  ToString(self: Enum,provider: IFormatProvider) -> str

  

   This method overload is obsolete; use System.Enum.ToString.

  

   provider: (obsolete)

   Returns: The string representation of the value of this instance.

  ToString(self: Enum) -> str

  

   Converts the value of this instance to its equivalent string representation.

   Returns: The string representation of the value of this instance.

  ToString(self: Enum,format: str,provider: IFormatProvider) -> str

  

   This method overload is obsolete; use System.Enum.ToString(System.String).

  

   format: A format specification.

   provider: (Obsolete.)

   Returns: The string representation of the value of this instance as specified by format.
  """
  pass
 @staticmethod
 def TryParse(value,*__args):
  """
  TryParse[TEnum](value: str,ignoreCase: bool) -> (bool,TEnum)

  TryParse[TEnum](value: str) -> (bool,TEnum)
  """
  pass
 def __and__(self,*args):
  """ __and__(self: object,other: object) -> object """
  pass
 def __eq__(self,*args):
  """ x.__eq__(y) <==> x==y """
  pass
 def __format__(self,*args):
  """ __format__(formattable: IFormattable,format: str) -> str """
  pass
 def __ge__(self,*args):
  pass
 def __gt__(self,*args):
  pass
 def __init__(self,*args):
  """ x.__init__(...) initializes x; see x.__class__.__doc__ for signaturex.__init__(...) initializes x; see x.__class__.__doc__ for signaturex.__init__(...) initializes x; see x.__class__.__doc__ for signature """
  pass
 def __invert__(self,*args):
  """ __invert__(self: object) -> object """
  pass
 def __le__(self,*args):
  pass
 def __lt__(self,*args):
  pass
 def __ne__(self,*args):
  pass
 def __nonzero__(self,*args):
  """ __nonzero__(self: object) -> bool """
  pass
 def __or__(self,*args):
  """ __or__(self: object,other: object) -> object """
  pass
 def __rand__(self,*args):
  """ __rand__(self: object,other: object) -> object """
  pass
 def __reduce_ex__(self,*args):
  pass
 def __repr__(self,*args):
  """ __repr__(self: object) -> str """
  pass
 def __ror__(self,*args):
  """ __ror__(self: object,other: object) -> object """
  pass
 def __rxor__(self,*args):
  """ __rxor__(self: object,other: object) -> object """
  pass
 def __str__(self,*args):
  pass
 def __xor__(self,*args):
  """ __xor__(self: object,other: object) -> object """
  pass



class DataGridTextBoxColumn(DataGridColumnStyle,IComponent,IDisposable,IDataGridColumnStyleEditingNotificationService):
 """
 Hosts a System.Windows.Forms.TextBox control in a cell of a System.Windows.Forms.DataGridColumnStyle for editing strings.

 

 DataGridTextBoxColumn()

 DataGridTextBoxColumn(prop: PropertyDescriptor)

 DataGridTextBoxColumn(prop: PropertyDescriptor,format: str)

 DataGridTextBoxColumn(prop: PropertyDescriptor,format: str,isDefault: bool)

 DataGridTextBoxColumn(prop: PropertyDescriptor,isDefault: bool)
 """
 def Abort(self,*args):
  """
  Abort(self: DataGridTextBoxColumn,rowNum: int)

   Initiates a request to interrupt an edit procedure.

  

   rowNum: The number of the row in which an edit operation is being interrupted.
  """
  pass
 def BeginUpdate(self,*args):
  """
  BeginUpdate(self: DataGridColumnStyle)

   Suspends the painting of the column until the System.Windows.Forms.DataGridColumnStyle.EndUpdate 

    method is called.
  """
  pass
 def CheckValidDataSource(self,*args):
  """
  CheckValidDataSource(self: DataGridColumnStyle,value: CurrencyManager)

   Throws an exception if the System.Windows.Forms.DataGrid does not have a valid data source,or 

    if this column is not mapped to a valid property in the data source.

  

  

   value: A System.Windows.Forms.CurrencyManager to check.
  """
  pass
 def ColumnStartedEditing(self,*args):
  """
  ColumnStartedEditing(self: DataGridColumnStyle,editingControl: Control)

   Informs the System.Windows.Forms.DataGrid that the user has begun editing the column.

  

   editingControl: The System.Windows.Forms.Control that hosted by the column.
  """
  pass
 def Commit(self,*args):
  """
  Commit(self: DataGridTextBoxColumn,dataSource: CurrencyManager,rowNum: int) -> bool

  

   Inititates a request to complete an editing procedure.

  

   dataSource: The System.Windows.Forms.CurrencyManager of the System.Windows.Forms.DataGrid control the column 

    belongs to.

  

   rowNum: The number of the edited row.

   Returns: true if the value was successfully committed; otherwise,false.
  """
  pass
 def ConcedeFocus(self,*args):
  """
  ConcedeFocus(self: DataGridTextBoxColumn)

   Informs the column that the focus is being conceded.
  """
  pass
 def CreateHeaderAccessibleObject(self,*args):
  """
  CreateHeaderAccessibleObject(self: DataGridColumnStyle) -> AccessibleObject

  

   Gets the System.Windows.Forms.AccessibleObject for the column.

   Returns: An System.Windows.Forms.AccessibleObject for the column.
  """
  pass
 def Dispose(self):
  """
  Dispose(self: Component,disposing: bool)

   Releases the unmanaged resources used by the System.ComponentModel.Component and optionally 

    releases the managed resources.

  

  

   disposing: true to release both managed and unmanaged resources; false to release only unmanaged resources.
  """
  pass
 def Edit(self,*args):
  """
  Edit(self: DataGridTextBoxColumn,source: CurrencyManager,rowNum: int,bounds: Rectangle,readOnly: bool,displayText: str,cellIsVisible: bool)

   Prepares a cell for editing.

  

   source: The System.Windows.Forms.CurrencyManager of the System.Windows.Forms.DataGrid control the column 

    belongs to.

  

   rowNum: The row number in this column being edited.

   bounds: The bounding System.Drawing.Rectangle in which the control is to be sited.

   readOnly: A value indicating whether the column is a read-only. true if the value is read-only; otherwise,

    false.

  

   displayText: The text to display in the control.

   cellIsVisible: A value indicating whether the cell is visible. true if the cell is visible; otherwise,false.

  Edit(self: DataGridColumnStyle,source: CurrencyManager,rowNum: int,bounds: Rectangle,readOnly: bool,displayText: str)

   Prepares the cell for editing using the specified System.Windows.Forms.CurrencyManager,row 

    number,and System.Drawing.Rectangle parameters.

  

  

   source: The System.Windows.Forms.CurrencyManager for the System.Windows.Forms.DataGridColumnStyle.

   rowNum: The row number in this column which is being edited.

   bounds: The System.Drawing.Rectangle in which the control is to be sited.

   readOnly: A value indicating whether the column is a read-only. true if the value is read-only; otherwise,

    false.

  

   displayText: The text to display in the control.

  Edit(self: DataGridColumnStyle,source: CurrencyManager,rowNum: int,bounds: Rectangle,readOnly: bool)

   Prepares a cell for editing.

  

   source: The System.Windows.Forms.CurrencyManager for the System.Windows.Forms.DataGridColumnStyle.

   rowNum: The row number to edit.

   bounds: The bounding System.Drawing.Rectangle in which the control is to be sited.

   readOnly: A value indicating whether the column is a read-only. true if the value is read-only; otherwise,

    false.
  """
  pass
 def EndEdit(self,*args):
  """
  EndEdit(self: DataGridTextBoxColumn)

   Ends an edit operation on the System.Windows.Forms.DataGridColumnStyle.
  """
  pass
 def EndUpdate(self,*args):
  """
  EndUpdate(self: DataGridColumnStyle)

   Resumes the painting of columns suspended by calling the 

    System.Windows.Forms.DataGridColumnStyle.BeginUpdate method.
  """
  pass
 def EnterNullValue(self,*args):
  """
  EnterNullValue(self: DataGridTextBoxColumn)

   Enters a System.DBNull.Value in the column.
  """
  pass
 def GetColumnValueAtRow(self,*args):
  """
  GetColumnValueAtRow(self: DataGridColumnStyle,source: CurrencyManager,rowNum: int) -> object

  

   Gets the value in the specified row from the specified System.Windows.Forms.CurrencyManager.

  

   source: The System.Windows.Forms.CurrencyManager containing the data.

   rowNum: The row number containing the data.

   Returns: An System.Object containing the value.
  """
  pass
 def GetMinimumHeight(self,*args):
  """
  GetMinimumHeight(self: DataGridTextBoxColumn) -> int

  

   Gets the height of a cell in a System.Windows.Forms.DataGridColumnStyle.

   Returns: The height of a cell.
  """
  pass
 def GetPreferredHeight(self,*args):
  """
  GetPreferredHeight(self: DataGridTextBoxColumn,g: Graphics,value: object) -> int

  

   Gets the height to be used in for automatically resizing columns.

  

   g: A System.Drawing.Graphics object used to draw shapes on the screen.

   value: The value to draw.

   Returns: The height the cells automatically resize to.
  """
  pass
 def GetPreferredSize(self,*args):
  """
  GetPreferredSize(self: DataGridTextBoxColumn,g: Graphics,value: object) -> Size

  

   Returns the optimum width and height of the cell in a specified row relative to the specified 

    value.

  

  

   g: A System.Drawing.Graphics object used to draw shapes on the screen.

   value: The value to draw.

   Returns: A System.Drawing.Size that contains the dimensions of the cell.
  """
  pass
 def GetService(self,*args):
  """
  GetService(self: Component,service: Type) -> object

  

   Returns an object that represents a service provided by the System.ComponentModel.Component or 

    by its System.ComponentModel.Container.

  

  

   service: A service provided by the System.ComponentModel.Component.

   Returns: An System.Object that represents a service provided by the System.ComponentModel.Component,or 

    null if the System.ComponentModel.Component does not provide the specified service.
  """
  pass
 def HideEditBox(self,*args):
  """
  HideEditBox(self: DataGridTextBoxColumn)

   Hides the System.Windows.Forms.DataGridTextBox control and moves the focus to the 

    System.Windows.Forms.DataGrid control.
  """
  pass
 def Invalidate(self,*args):
  """
  Invalidate(self: DataGridColumnStyle)

   Redraws the column and causes a paint message to be sent to the control.
  """
  pass
 def MemberwiseClone(self,*args):
  """
  MemberwiseClone(self: MarshalByRefObject,cloneIdentity: bool) -> MarshalByRefObject

  

   Creates a shallow copy of the current System.MarshalByRefObject object.

  

   cloneIdentity: false to delete the current System.MarshalByRefObject object's identity,which will cause the 

    object to be assigned a new identity when it is marshaled across a remoting boundary. A value of 

    false is usually appropriate. true to copy the current System.MarshalByRefObject object's 

    identity to its clone,which will cause remoting client calls to be routed to the remote server 

    object.

  

   Returns: A shallow copy of the current System.MarshalByRefObject object.

  MemberwiseClone(self: object) -> object

  

   Creates a shallow copy of the current System.Object.

   Returns: A shallow copy of the current System.Object.
  """
  pass
 def Paint(self,*args):
  """
  Paint(self: DataGridTextBoxColumn,g: Graphics,bounds: Rectangle,source: CurrencyManager,rowNum: int,backBrush: Brush,foreBrush: Brush,alignToRight: bool)

   Paints a System.Windows.Forms.DataGridColumnStyle with the specified System.Drawing.Graphics,

    System.Drawing.Rectangle,System.Windows.Forms.CurrencyManager,row number,

    System.Drawing.Brush,and foreground color.

  

  

   g: The System.Drawing.Graphics object to draw to.

   bounds: The bounding System.Drawing.Rectangle to paint into.

   source: The System.Windows.Forms.CurrencyManager of the System.Windows.Forms.DataGrid the that contains 

    the column.

  

   rowNum: The number of the row in the underlying data table.

   backBrush: A System.Drawing.Brush that paints the background.

   foreBrush: A System.Drawing.Brush that paints the foreground color.

   alignToRight: A value indicating whether to align the column's content to the right. true if the content 

    should be aligned to the right; otherwise,false.

  

  Paint(self: DataGridTextBoxColumn,g: Graphics,bounds: Rectangle,source: CurrencyManager,rowNum: int,alignToRight: bool)

   Paints a System.Windows.Forms.DataGridColumnStyle with the specified System.Drawing.Graphics,

    System.Drawing.Rectangle,System.Windows.Forms.CurrencyManager,row number,and alignment.

  

  

   g: The System.Drawing.Graphics object to draw to.

   bounds: The bounding System.Drawing.Rectangle to paint into.

   source: The System.Windows.Forms.CurrencyManager of the System.Windows.Forms.DataGrid the that contains 

    the column.

  

   rowNum: The number of the row in the underlying data table.

   alignToRight: A value indicating whether to align the column's content to the right. true if the content 

    should be aligned to the right; otherwise,false.

  

  Paint(self: DataGridTextBoxColumn,g: Graphics,bounds: Rectangle,source: CurrencyManager,rowNum: int)

   Paints the a System.Windows.Forms.DataGridColumnStyle with the specified 

    System.Drawing.Graphics,System.Drawing.Rectangle,System.Windows.Forms.CurrencyManager,and row 

    number.

  

  

   g: The System.Drawing.Graphics object to draw to.

   bounds: The bounding System.Drawing.Rectangle to paint into.

   source: The System.Windows.Forms.CurrencyManager of the System.Windows.Forms.DataGrid the that contains 

    the column.

  

   rowNum: The number of the row in the underlying data table.
  """
  pass
 def PaintText(self,*args):
  """
  PaintText(self: DataGridTextBoxColumn,g: Graphics,textBounds: Rectangle,text: str,backBrush: Brush,foreBrush: Brush,alignToRight: bool)

   Draws the text and rectangle at the specified location with the specified colors and alignment.

  

   g: A System.Drawing.Graphics object used to draw the string.

   textBounds: A System.Drawing.Rectangle which contains the boundary data of the rectangle.

   text: The string to be drawn to the screen.

   backBrush: A System.Drawing.Brush that determines the rectangle's background color

   foreBrush: A System.Drawing.Brush that determines the rectangles foreground color.

   alignToRight: A value indicating whether the text is right-aligned.

  PaintText(self: DataGridTextBoxColumn,g: Graphics,bounds: Rectangle,text: str,alignToRight: bool)

   Draws the text and rectangle at the given location with the specified alignment.

  

   g: A System.Drawing.Graphics object used to draw the string.

   bounds: A System.Drawing.Rectangle which contains the boundary data of the rectangle.

   text: The string to be drawn to the screen.

   alignToRight: A value indicating whether the text is right-aligned.
  """
  pass
 def ReleaseHostedControl(self,*args):
  """
  ReleaseHostedControl(self: DataGridTextBoxColumn)

   Removes the reference that the System.Windows.Forms.DataGrid holds to the control used to edit 

    data.
  """
  pass
 def SetColumnValueAtRow(self,*args):
  """
  SetColumnValueAtRow(self: DataGridColumnStyle,source: CurrencyManager,rowNum: int,value: object)

   Sets the value in a specified row with the value from a specified 

    System.Windows.Forms.CurrencyManager.

  

  

   source: A System.Windows.Forms.CurrencyManager associated with the 

    System.Windows.Forms.DataGridColumnStyle.

  

   rowNum: The number of the row.

   value: The value to set.
  """
  pass
 def SetDataGrid(self,*args):
  """
  SetDataGrid(self: DataGridColumnStyle,value: DataGrid)

   Sets the System.Windows.Forms.DataGrid control that this column belongs to.

  

   value: The System.Windows.Forms.DataGrid control that this column belongs to.
  """
  pass
 def SetDataGridInColumn(self,*args):
  """
  SetDataGridInColumn(self: DataGridTextBoxColumn,value: DataGrid)

   Adds a System.Windows.Forms.TextBox control to the System.Windows.Forms.DataGrid control's 

    System.Windows.Forms.Control.ControlCollection.

  

  

   value: The System.Windows.Forms.DataGrid control the System.Windows.Forms.TextBox control is added to.
  """
  pass
 def UpdateUI(self,*args):
  """
  UpdateUI(self: DataGridTextBoxColumn,source: CurrencyManager,rowNum: int,displayText: str)

   Updates the user interface.

  

   source: The System.Windows.Forms.CurrencyManager that supplies the data.

   rowNum: The index of the row to update.

   displayText: The text that will be displayed in the cell.
  """
  pass
 def __enter__(self,*args):
  """
  __enter__(self: IDisposable) -> object

  

   Provides the implementation of __enter__ for objects which implement IDisposable.
  """
  pass
 def __exit__(self,*args):
  """
  __exit__(self: IDisposable,exc_type: object,exc_value: object,exc_back: object)

   Provides the implementation of __exit__ for objects which implement IDisposable.
  """
  pass
 def __init__(self,*args):
  """ x.__init__(...) initializes x; see x.__class__.__doc__ for signaturex.__init__(...) initializes x; see x.__class__.__doc__ for signaturex.__init__(...) initializes x; see x.__class__.__doc__ for signature """
  pass
 @staticmethod
 def __new__(self,prop=None,*__args):
  """
  __new__(cls: type)

  __new__(cls: type,prop: PropertyDescriptor)

  __new__(cls: type,prop: PropertyDescriptor,format: str)

  __new__(cls: type,prop: PropertyDescriptor,format: str,isDefault: bool)

  __new__(cls: type,prop: PropertyDescriptor,isDefault: bool)
  """
  pass
 def __str__(self,*args):
  pass
 CanRaiseEvents=property(lambda self: object(),lambda self,v: None,lambda self: None)
 """Gets a value indicating whether the component can raise an event.



"""

 DesignMode=property(lambda self: object(),lambda self,v: None,lambda self: None)
 """Gets a value that indicates whether the System.ComponentModel.Component is currently in design mode.



"""

 Events=property(lambda self: object(),lambda self,v: None,lambda self: None)
 """Gets the list of event handlers that are attached to this System.ComponentModel.Component.



"""

 FontHeight=property(lambda self: object(),lambda self,v: None,lambda self: None)
 """Gets the height of the column's font.



"""

 Format=property(lambda self: object(),lambda self,v: None,lambda self: None)
 """Gets or sets the character(s) that specify how text is formatted.



Get: Format(self: DataGridTextBoxColumn) -> str



Set: Format(self: DataGridTextBoxColumn)=value

"""

 FormatInfo=property(lambda self: object(),lambda self,v: None,lambda self: None)
 """Gets or sets the culture specific information used to determine how values are formatted.



Get: FormatInfo(self: DataGridTextBoxColumn) -> IFormatProvider



Set: FormatInfo(self: DataGridTextBoxColumn)=value

"""

 PropertyDescriptor=property(lambda self: object(),lambda self,v: None,lambda self: None)
 """Gets or sets the System.ComponentModel.PropertyDescriptor for the System.Windows.Forms.DataGridTextBoxColumn.



Set: PropertyDescriptor(self: DataGridTextBoxColumn)=value

"""

 ReadOnly=property(lambda self: object(),lambda self,v: None,lambda self: None)
 """Sets a value indicating whether the text box column is read-only.



Get: ReadOnly(self: DataGridTextBoxColumn) -> bool



Set: ReadOnly(self: DataGridTextBoxColumn)=value

"""

 TextBox=property(lambda self: object(),lambda self,v: None,lambda self: None)
 """Gets the hosted System.Windows.Forms.TextBox control.



Get: TextBox(self: DataGridTextBoxColumn) -> TextBox



"""





class FUNCDESC(object):
 """ Use System.Runtime.InteropServices.ComTypes.FUNCDESC instead. """
 callconv=None
 cParams=None
 cParamsOpt=None
 cScodes=None
 elemdescFunc=None
 funckind=None
 invkind=None
 lprgelemdescParam=None
 lprgscode=None
 memid=None
 oVft=None
 wFuncFlags=None




class ImportedFromTypeLibAttribute(Attribute,_Attribute):
 """
 Indicates that the types defined within an assembly were originally defined in a type library.

 

 ImportedFromTypeLibAttribute(tlbFile: str)
 """
 def __init__(self,*args):
  """ x.__init__(...) initializes x; see x.__class__.__doc__ for signaturex.__init__(...) initializes x; see x.__class__.__doc__ for signaturex.__init__(...) initializes x; see x.__class__.__doc__ for signature """
  pass
 @staticmethod
 def __new__(self,tlbFile):
  """ __new__(cls: type,tlbFile: str) """
  pass
 Value=property(lambda self: object(),lambda self,v: None,lambda self: None)
 """Gets the name of the original type library file.



Get: Value(self: ImportedFromTypeLibAttribute) -> str



"""





class VariantWrapper(object):
 """
 Marshals data of type VT_VARIANT | VT_BYREF from managed to unmanaged code. This class cannot be inherited.

 

 VariantWrapper(obj: object)
 """
 @staticmethod
 def __new__(self,obj):
  """ __new__(cls: type,obj: object) """
  pass
 WrappedObject=property(lambda self: object(),lambda self,v: None,lambda self: None)
 """Gets the object wrapped by the System.Runtime.InteropServices.VariantWrapper object.



Get: WrappedObject(self: VariantWrapper) -> object



"""





class WarningException(SystemException,ISerializable,_Exception):
 """
 Specifies an exception that is handled as a warning instead of an error.

 

 WarningException()

 WarningException(message: str)

 WarningException(message: str,helpUrl: str)

 WarningException(message: str,innerException: Exception)

 WarningException(message: str,helpUrl: str,helpTopic: str)
 """
 def add_SerializeObjectState(self,*args):
  """ add_SerializeObjectState(self: Exception,value: EventHandler[SafeSerializationEventArgs]) """
  pass
 def GetObjectData(self,info,context):
  """
  GetObjectData(self: WarningException,info: SerializationInfo,context: StreamingContext)

   Sets the System.Runtime.Serialization.SerializationInfo with the parameter name and additional 

    exception information.

  

  

   info: Stores the data that was being used to serialize or deserialize the object that the 

    System.ComponentModel.Design.Serialization.CodeDomSerializer was serializing or deserializing.

  

   context: Describes the source and destination of the stream that generated the exception,as well as a 

    means for serialization to retain that context and an additional caller-defined context.
  """
  pass
 def remove_SerializeObjectState(self,*args):
  """ remove_SerializeObjectState(self: Exception,value: EventHandler[SafeSerializationEventArgs]) """
  pass
 def __init__(self,*args):
  """ x.__init__(...) initializes x; see x.__class__.__doc__ for signaturex.__init__(...) initializes x; see x.__class__.__doc__ for signaturex.__init__(...) initializes x; see x.__class__.__doc__ for signature """
  pass
 @staticmethod
 def __new__(self,message=None,*__args):
  """
  __new__(cls: type)

  __new__(cls: type,message: str)

  __new__(cls: type,message: str,helpUrl: str)

  __new__(cls: type,message: str,innerException: Exception)

  __new__(cls: type,message: str,helpUrl: str,helpTopic: str)

  __new__(cls: type,info: SerializationInfo,context: StreamingContext)
  """
  pass
 def __reduce_ex__(self,*args):
  pass
 def __str__(self,*args):
  pass
 HelpTopic=property(lambda self: object(),lambda self,v: None,lambda self: None)
 """Gets the Help topic associated with the warning.



Get: HelpTopic(self: WarningException) -> str



"""

 HelpUrl=property(lambda self: object(),lambda self,v: None,lambda self: None)
 """Gets the Help file associated with the warning.



Get: HelpUrl(self: WarningException) -> str



"""





class CounterCreationData(object):
 """
 Defines the counter type,name,and Help string for a custom counter.

 

 CounterCreationData(counterName: str,counterHelp: str,counterType: PerformanceCounterType)

 CounterCreationData()
 """
 @staticmethod
 def __new__(self,counterName=None,counterHelp=None,counterType=None):
  """
  __new__(cls: type)

  __new__(cls: type,counterName: str,counterHelp: str,counterType: PerformanceCounterType)
  """
  pass
 CounterHelp=property(lambda self: object(),lambda self,v: None,lambda self: None)
 """Gets or sets the custom counter's description.



Get: CounterHelp(self: CounterCreationData) -> str



Set: CounterHelp(self: CounterCreationData)=value

"""

 CounterName=property(lambda self: object(),lambda self,v: None,lambda self: None)
 """Gets or sets the name of the custom counter.



Get: CounterName(self: CounterCreationData) -> str



Set: CounterName(self: CounterCreationData)=value

"""

 CounterType=property(lambda self: object(),lambda self,v: None,lambda self: None)
 """Gets or sets the performance counter type of the custom counter.



Get: CounterType(self: CounterCreationData) -> PerformanceCounterType



Set: CounterType(self: CounterCreationData)=value

"""





class ToolStripRenderEventArgs(EventArgs):
 """
 Provides data for the System.Windows.Forms.ToolStripRenderer.OnRenderImageMargin(System.Windows.Forms.ToolStripRenderEventArgs),System.Windows.Forms.ToolStripRenderer.OnRenderToolStripBorder(System.Windows.Forms.ToolStripRenderEventArgs),and System.Windows.Forms.ToolStripRenderer.OnRenderToolStripBackground(System.Windows.Forms.ToolStripRenderEventArgs) methods.

 

 ToolStripRenderEventArgs(g: Graphics,toolStrip: ToolStrip)

 ToolStripRenderEventArgs(g: Graphics,toolStrip: ToolStrip,affectedBounds: Rectangle,backColor: Color)
 """
 @staticmethod
 def __new__(self,g,toolStrip,affectedBounds=None,backColor=None):
  """
  __new__(cls: type,g: Graphics,toolStrip: ToolStrip)

  __new__(cls: type,g: Graphics,toolStrip: ToolStrip,affectedBounds: Rectangle,backColor: Color)
  """
  pass
 AffectedBounds=property(lambda self: object(),lambda self,v: None,lambda self: None)
 """Gets the System.Drawing.Rectangle representing the bounds of the area to be painted.



Get: AffectedBounds(self: ToolStripRenderEventArgs) -> Rectangle



"""

 BackColor=property(lambda self: object(),lambda self,v: None,lambda self: None)
 """Gets the System.Drawing.Color that the background of the System.Windows.Forms.ToolStrip is painted with.



Get: BackColor(self: ToolStripRenderEventArgs) -> Color



"""

 ConnectedArea=property(lambda self: object(),lambda self,v: None,lambda self: None)
 """Gets the System.Drawing.Rectangle representing the overlap area between a System.Windows.Forms.ToolStripDropDown and its System.Windows.Forms.ToolStripDropDown.OwnerItem.



Get: ConnectedArea(self: ToolStripRenderEventArgs) -> Rectangle



"""

 Graphics=property(lambda self: object(),lambda self,v: None,lambda self: None)
 """Gets the System.Drawing.Graphics used to paint.



Get: Graphics(self: ToolStripRenderEventArgs) -> Graphics



"""

 ToolStrip=property(lambda self: object(),lambda self,v: None,lambda self: None)
 """Gets the System.Windows.Forms.ToolStrip to be painted.



Get: ToolStrip(self: ToolStripRenderEventArgs) -> ToolStrip



"""


